// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/storage.proto

package storagepb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	status "google.golang.org/genproto/googleapis/rpc/status"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// An enum to indicate how to interpret missing values of fields that are
// present in user schema but missing in rows. A missing value can represent a
// NULL or a column default value defined in BigQuery table schema.
type AppendRowsRequest_MissingValueInterpretation int32

const (
	// Invalid missing value interpretation. Requests with this value will be
	// rejected.
	AppendRowsRequest_MISSING_VALUE_INTERPRETATION_UNSPECIFIED AppendRowsRequest_MissingValueInterpretation = 0
	// Missing value is interpreted as NULL.
	AppendRowsRequest_NULL_VALUE AppendRowsRequest_MissingValueInterpretation = 1
	// Missing value is interpreted as column default value if declared in the
	// table schema, NULL otherwise.
	AppendRowsRequest_DEFAULT_VALUE AppendRowsRequest_MissingValueInterpretation = 2
)

// Enum value maps for AppendRowsRequest_MissingValueInterpretation.
var (
	AppendRowsRequest_MissingValueInterpretation_name = map[int32]string{
		0: "MISSING_VALUE_INTERPRETATION_UNSPECIFIED",
		1: "NULL_VALUE",
		2: "DEFAULT_VALUE",
	}
	AppendRowsRequest_MissingValueInterpretation_value = map[string]int32{
		"MISSING_VALUE_INTERPRETATION_UNSPECIFIED": 0,
		"NULL_VALUE":    1,
		"DEFAULT_VALUE": 2,
	}
)

func (x AppendRowsRequest_MissingValueInterpretation) Enum() *AppendRowsRequest_MissingValueInterpretation {
	p := new(AppendRowsRequest_MissingValueInterpretation)
	*p = x
	return p
}

func (x AppendRowsRequest_MissingValueInterpretation) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AppendRowsRequest_MissingValueInterpretation) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[0].Descriptor()
}

func (AppendRowsRequest_MissingValueInterpretation) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[0]
}

func (x AppendRowsRequest_MissingValueInterpretation) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AppendRowsRequest_MissingValueInterpretation.Descriptor instead.
func (AppendRowsRequest_MissingValueInterpretation) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{8, 0}
}

// Error code for `StorageError`.
type StorageError_StorageErrorCode int32

const (
	// Default error.
	StorageError_STORAGE_ERROR_CODE_UNSPECIFIED StorageError_StorageErrorCode = 0
	// Table is not found in the system.
	StorageError_TABLE_NOT_FOUND StorageError_StorageErrorCode = 1
	// Stream is already committed.
	StorageError_STREAM_ALREADY_COMMITTED StorageError_StorageErrorCode = 2
	// Stream is not found.
	StorageError_STREAM_NOT_FOUND StorageError_StorageErrorCode = 3
	// Invalid Stream type.
	// For example, you try to commit a stream that is not pending.
	StorageError_INVALID_STREAM_TYPE StorageError_StorageErrorCode = 4
	// Invalid Stream state.
	// For example, you try to commit a stream that is not finalized or is
	// garbaged.
	StorageError_INVALID_STREAM_STATE StorageError_StorageErrorCode = 5
	// Stream is finalized.
	StorageError_STREAM_FINALIZED StorageError_StorageErrorCode = 6
	// There is a schema mismatch and it is caused by user schema has extra
	// field than bigquery schema.
	StorageError_SCHEMA_MISMATCH_EXTRA_FIELDS StorageError_StorageErrorCode = 7
	// Offset already exists.
	StorageError_OFFSET_ALREADY_EXISTS StorageError_StorageErrorCode = 8
	// Offset out of range.
	StorageError_OFFSET_OUT_OF_RANGE StorageError_StorageErrorCode = 9
	// Customer-managed encryption key (CMEK) not provided for CMEK-enabled
	// data.
	StorageError_CMEK_NOT_PROVIDED StorageError_StorageErrorCode = 10
	// Customer-managed encryption key (CMEK) was incorrectly provided.
	StorageError_INVALID_CMEK_PROVIDED StorageError_StorageErrorCode = 11
	// There is an encryption error while using customer-managed encryption key.
	StorageError_CMEK_ENCRYPTION_ERROR StorageError_StorageErrorCode = 12
	// Key Management Service (KMS) service returned an error, which can be
	// retried.
	StorageError_KMS_SERVICE_ERROR StorageError_StorageErrorCode = 13
	// Permission denied while using customer-managed encryption key.
	StorageError_KMS_PERMISSION_DENIED StorageError_StorageErrorCode = 14
)

// Enum value maps for StorageError_StorageErrorCode.
var (
	StorageError_StorageErrorCode_name = map[int32]string{
		0:  "STORAGE_ERROR_CODE_UNSPECIFIED",
		1:  "TABLE_NOT_FOUND",
		2:  "STREAM_ALREADY_COMMITTED",
		3:  "STREAM_NOT_FOUND",
		4:  "INVALID_STREAM_TYPE",
		5:  "INVALID_STREAM_STATE",
		6:  "STREAM_FINALIZED",
		7:  "SCHEMA_MISMATCH_EXTRA_FIELDS",
		8:  "OFFSET_ALREADY_EXISTS",
		9:  "OFFSET_OUT_OF_RANGE",
		10: "CMEK_NOT_PROVIDED",
		11: "INVALID_CMEK_PROVIDED",
		12: "CMEK_ENCRYPTION_ERROR",
		13: "KMS_SERVICE_ERROR",
		14: "KMS_PERMISSION_DENIED",
	}
	StorageError_StorageErrorCode_value = map[string]int32{
		"STORAGE_ERROR_CODE_UNSPECIFIED": 0,
		"TABLE_NOT_FOUND":                1,
		"STREAM_ALREADY_COMMITTED":       2,
		"STREAM_NOT_FOUND":               3,
		"INVALID_STREAM_TYPE":            4,
		"INVALID_STREAM_STATE":           5,
		"STREAM_FINALIZED":               6,
		"SCHEMA_MISMATCH_EXTRA_FIELDS":   7,
		"OFFSET_ALREADY_EXISTS":          8,
		"OFFSET_OUT_OF_RANGE":            9,
		"CMEK_NOT_PROVIDED":              10,
		"INVALID_CMEK_PROVIDED":          11,
		"CMEK_ENCRYPTION_ERROR":          12,
		"KMS_SERVICE_ERROR":              13,
		"KMS_PERMISSION_DENIED":          14,
	}
)

func (x StorageError_StorageErrorCode) Enum() *StorageError_StorageErrorCode {
	p := new(StorageError_StorageErrorCode)
	*p = x
	return p
}

func (x StorageError_StorageErrorCode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (StorageError_StorageErrorCode) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[1].Descriptor()
}

func (StorageError_StorageErrorCode) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[1]
}

func (x StorageError_StorageErrorCode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use StorageError_StorageErrorCode.Descriptor instead.
func (StorageError_StorageErrorCode) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{17, 0}
}

// Error code for `RowError`.
type RowError_RowErrorCode int32

const (
	// Default error.
	RowError_ROW_ERROR_CODE_UNSPECIFIED RowError_RowErrorCode = 0
	// One or more fields in the row has errors.
	RowError_FIELDS_ERROR RowError_RowErrorCode = 1
)

// Enum value maps for RowError_RowErrorCode.
var (
	RowError_RowErrorCode_name = map[int32]string{
		0: "ROW_ERROR_CODE_UNSPECIFIED",
		1: "FIELDS_ERROR",
	}
	RowError_RowErrorCode_value = map[string]int32{
		"ROW_ERROR_CODE_UNSPECIFIED": 0,
		"FIELDS_ERROR":               1,
	}
)

func (x RowError_RowErrorCode) Enum() *RowError_RowErrorCode {
	p := new(RowError_RowErrorCode)
	*p = x
	return p
}

func (x RowError_RowErrorCode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RowError_RowErrorCode) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[2].Descriptor()
}

func (RowError_RowErrorCode) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes[2]
}

func (x RowError_RowErrorCode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use RowError_RowErrorCode.Descriptor instead.
func (RowError_RowErrorCode) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{18, 0}
}

// Request message for `CreateReadSession`.
type CreateReadSessionRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The request project that owns the session, in the form of
	// `projects/{project_id}`.
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Required. Session to be created.
	ReadSession *ReadSession `protobuf:"bytes,2,opt,name=read_session,json=readSession,proto3" json:"read_session,omitempty"`
	// Max initial number of streams. If unset or zero, the server will
	// provide a value of streams so as to produce reasonable throughput. Must be
	// non-negative. The number of streams may be lower than the requested number,
	// depending on the amount parallelism that is reasonable for the table.
	// There is a default system max limit of 1,000.
	//
	// This must be greater than or equal to preferred_min_stream_count.
	// Typically, clients should either leave this unset to let the system to
	// determine an upper bound OR set this a size for the maximum "units of work"
	// it can gracefully handle.
	MaxStreamCount int32 `protobuf:"varint,3,opt,name=max_stream_count,json=maxStreamCount,proto3" json:"max_stream_count,omitempty"`
	// The minimum preferred stream count. This parameter can be used to inform
	// the service that there is a desired lower bound on the number of streams.
	// This is typically a target parallelism of the client (e.g. a Spark
	// cluster with N-workers would set this to a low multiple of N to ensure
	// good cluster utilization).
	//
	// The system will make a best effort to provide at least this number of
	// streams, but in some cases might provide less.
	PreferredMinStreamCount int32 `protobuf:"varint,4,opt,name=preferred_min_stream_count,json=preferredMinStreamCount,proto3" json:"preferred_min_stream_count,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *CreateReadSessionRequest) Reset() {
	*x = CreateReadSessionRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateReadSessionRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateReadSessionRequest) ProtoMessage() {}

func (x *CreateReadSessionRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateReadSessionRequest.ProtoReflect.Descriptor instead.
func (*CreateReadSessionRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{0}
}

func (x *CreateReadSessionRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateReadSessionRequest) GetReadSession() *ReadSession {
	if x != nil {
		return x.ReadSession
	}
	return nil
}

func (x *CreateReadSessionRequest) GetMaxStreamCount() int32 {
	if x != nil {
		return x.MaxStreamCount
	}
	return 0
}

func (x *CreateReadSessionRequest) GetPreferredMinStreamCount() int32 {
	if x != nil {
		return x.PreferredMinStreamCount
	}
	return 0
}

// Request message for `ReadRows`.
type ReadRowsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Stream to read rows from.
	ReadStream string `protobuf:"bytes,1,opt,name=read_stream,json=readStream,proto3" json:"read_stream,omitempty"`
	// The offset requested must be less than the last row read from Read.
	// Requesting a larger offset is undefined. If not specified, start reading
	// from offset zero.
	Offset        int64 `protobuf:"varint,2,opt,name=offset,proto3" json:"offset,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ReadRowsRequest) Reset() {
	*x = ReadRowsRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadRowsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadRowsRequest) ProtoMessage() {}

func (x *ReadRowsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadRowsRequest.ProtoReflect.Descriptor instead.
func (*ReadRowsRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{1}
}

func (x *ReadRowsRequest) GetReadStream() string {
	if x != nil {
		return x.ReadStream
	}
	return ""
}

func (x *ReadRowsRequest) GetOffset() int64 {
	if x != nil {
		return x.Offset
	}
	return 0
}

// Information on if the current connection is being throttled.
type ThrottleState struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// How much this connection is being throttled. Zero means no throttling,
	// 100 means fully throttled.
	ThrottlePercent int32 `protobuf:"varint,1,opt,name=throttle_percent,json=throttlePercent,proto3" json:"throttle_percent,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *ThrottleState) Reset() {
	*x = ThrottleState{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ThrottleState) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ThrottleState) ProtoMessage() {}

func (x *ThrottleState) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ThrottleState.ProtoReflect.Descriptor instead.
func (*ThrottleState) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{2}
}

func (x *ThrottleState) GetThrottlePercent() int32 {
	if x != nil {
		return x.ThrottlePercent
	}
	return 0
}

// Estimated stream statistics for a given read Stream.
type StreamStats struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Represents the progress of the current stream.
	Progress      *StreamStats_Progress `protobuf:"bytes,2,opt,name=progress,proto3" json:"progress,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamStats) Reset() {
	*x = StreamStats{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamStats) ProtoMessage() {}

func (x *StreamStats) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamStats.ProtoReflect.Descriptor instead.
func (*StreamStats) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{3}
}

func (x *StreamStats) GetProgress() *StreamStats_Progress {
	if x != nil {
		return x.Progress
	}
	return nil
}

// Response from calling `ReadRows` may include row data, progress and
// throttling information.
type ReadRowsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Row data is returned in format specified during session creation.
	//
	// Types that are valid to be assigned to Rows:
	//
	//	*ReadRowsResponse_AvroRows
	//	*ReadRowsResponse_ArrowRecordBatch
	Rows isReadRowsResponse_Rows `protobuf_oneof:"rows"`
	// Number of serialized rows in the rows block.
	RowCount int64 `protobuf:"varint,6,opt,name=row_count,json=rowCount,proto3" json:"row_count,omitempty"`
	// Statistics for the stream.
	Stats *StreamStats `protobuf:"bytes,2,opt,name=stats,proto3" json:"stats,omitempty"`
	// Throttling state. If unset, the latest response still describes
	// the current throttling status.
	ThrottleState *ThrottleState `protobuf:"bytes,5,opt,name=throttle_state,json=throttleState,proto3" json:"throttle_state,omitempty"`
	// The schema for the read. If read_options.selected_fields is set, the
	// schema may be different from the table schema as it will only contain
	// the selected fields. This schema is equivalent to the one returned by
	// CreateSession. This field is only populated in the first ReadRowsResponse
	// RPC.
	//
	// Types that are valid to be assigned to Schema:
	//
	//	*ReadRowsResponse_AvroSchema
	//	*ReadRowsResponse_ArrowSchema
	Schema isReadRowsResponse_Schema `protobuf_oneof:"schema"`
	// Optional. If the row data in this ReadRowsResponse is compressed, then
	// uncompressed byte size is the original size of the uncompressed row data.
	// If it is set to a value greater than 0, then decompress into a buffer of
	// size uncompressed_byte_size using the compression codec that was requested
	// during session creation time and which is specified in
	// TableReadOptions.response_compression_codec in ReadSession.
	// This value is not set if no response_compression_codec was not requested
	// and it is -1 if the requested compression would not have reduced the size
	// of this ReadRowsResponse's row data. This attempts to match Apache Arrow's
	// behavior described here https://github.com/apache/arrow/issues/15102 where
	// the uncompressed length may be set to -1 to indicate that the data that
	// follows is not compressed, which can be useful for cases where compression
	// does not yield appreciable savings. When uncompressed_byte_size is not
	// greater than 0, the client should skip decompression.
	UncompressedByteSize *int64 `protobuf:"varint,9,opt,name=uncompressed_byte_size,json=uncompressedByteSize,proto3,oneof" json:"uncompressed_byte_size,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *ReadRowsResponse) Reset() {
	*x = ReadRowsResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadRowsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadRowsResponse) ProtoMessage() {}

func (x *ReadRowsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadRowsResponse.ProtoReflect.Descriptor instead.
func (*ReadRowsResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{4}
}

func (x *ReadRowsResponse) GetRows() isReadRowsResponse_Rows {
	if x != nil {
		return x.Rows
	}
	return nil
}

func (x *ReadRowsResponse) GetAvroRows() *AvroRows {
	if x != nil {
		if x, ok := x.Rows.(*ReadRowsResponse_AvroRows); ok {
			return x.AvroRows
		}
	}
	return nil
}

func (x *ReadRowsResponse) GetArrowRecordBatch() *ArrowRecordBatch {
	if x != nil {
		if x, ok := x.Rows.(*ReadRowsResponse_ArrowRecordBatch); ok {
			return x.ArrowRecordBatch
		}
	}
	return nil
}

func (x *ReadRowsResponse) GetRowCount() int64 {
	if x != nil {
		return x.RowCount
	}
	return 0
}

func (x *ReadRowsResponse) GetStats() *StreamStats {
	if x != nil {
		return x.Stats
	}
	return nil
}

func (x *ReadRowsResponse) GetThrottleState() *ThrottleState {
	if x != nil {
		return x.ThrottleState
	}
	return nil
}

func (x *ReadRowsResponse) GetSchema() isReadRowsResponse_Schema {
	if x != nil {
		return x.Schema
	}
	return nil
}

func (x *ReadRowsResponse) GetAvroSchema() *AvroSchema {
	if x != nil {
		if x, ok := x.Schema.(*ReadRowsResponse_AvroSchema); ok {
			return x.AvroSchema
		}
	}
	return nil
}

func (x *ReadRowsResponse) GetArrowSchema() *ArrowSchema {
	if x != nil {
		if x, ok := x.Schema.(*ReadRowsResponse_ArrowSchema); ok {
			return x.ArrowSchema
		}
	}
	return nil
}

func (x *ReadRowsResponse) GetUncompressedByteSize() int64 {
	if x != nil && x.UncompressedByteSize != nil {
		return *x.UncompressedByteSize
	}
	return 0
}

type isReadRowsResponse_Rows interface {
	isReadRowsResponse_Rows()
}

type ReadRowsResponse_AvroRows struct {
	// Serialized row data in AVRO format.
	AvroRows *AvroRows `protobuf:"bytes,3,opt,name=avro_rows,json=avroRows,proto3,oneof"`
}

type ReadRowsResponse_ArrowRecordBatch struct {
	// Serialized row data in Arrow RecordBatch format.
	ArrowRecordBatch *ArrowRecordBatch `protobuf:"bytes,4,opt,name=arrow_record_batch,json=arrowRecordBatch,proto3,oneof"`
}

func (*ReadRowsResponse_AvroRows) isReadRowsResponse_Rows() {}

func (*ReadRowsResponse_ArrowRecordBatch) isReadRowsResponse_Rows() {}

type isReadRowsResponse_Schema interface {
	isReadRowsResponse_Schema()
}

type ReadRowsResponse_AvroSchema struct {
	// Output only. Avro schema.
	AvroSchema *AvroSchema `protobuf:"bytes,7,opt,name=avro_schema,json=avroSchema,proto3,oneof"`
}

type ReadRowsResponse_ArrowSchema struct {
	// Output only. Arrow schema.
	ArrowSchema *ArrowSchema `protobuf:"bytes,8,opt,name=arrow_schema,json=arrowSchema,proto3,oneof"`
}

func (*ReadRowsResponse_AvroSchema) isReadRowsResponse_Schema() {}

func (*ReadRowsResponse_ArrowSchema) isReadRowsResponse_Schema() {}

// Request message for `SplitReadStream`.
type SplitReadStreamRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the stream to split.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// A value in the range (0.0, 1.0) that specifies the fractional point at
	// which the original stream should be split. The actual split point is
	// evaluated on pre-filtered rows, so if a filter is provided, then there is
	// no guarantee that the division of the rows between the new child streams
	// will be proportional to this fractional value. Additionally, because the
	// server-side unit for assigning data is collections of rows, this fraction
	// will always map to a data storage boundary on the server side.
	Fraction      float64 `protobuf:"fixed64,2,opt,name=fraction,proto3" json:"fraction,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SplitReadStreamRequest) Reset() {
	*x = SplitReadStreamRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SplitReadStreamRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SplitReadStreamRequest) ProtoMessage() {}

func (x *SplitReadStreamRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SplitReadStreamRequest.ProtoReflect.Descriptor instead.
func (*SplitReadStreamRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{5}
}

func (x *SplitReadStreamRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *SplitReadStreamRequest) GetFraction() float64 {
	if x != nil {
		return x.Fraction
	}
	return 0
}

// Response message for `SplitReadStream`.
type SplitReadStreamResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Primary stream, which contains the beginning portion of
	// |original_stream|. An empty value indicates that the original stream can no
	// longer be split.
	PrimaryStream *ReadStream `protobuf:"bytes,1,opt,name=primary_stream,json=primaryStream,proto3" json:"primary_stream,omitempty"`
	// Remainder stream, which contains the tail of |original_stream|. An empty
	// value indicates that the original stream can no longer be split.
	RemainderStream *ReadStream `protobuf:"bytes,2,opt,name=remainder_stream,json=remainderStream,proto3" json:"remainder_stream,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *SplitReadStreamResponse) Reset() {
	*x = SplitReadStreamResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SplitReadStreamResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SplitReadStreamResponse) ProtoMessage() {}

func (x *SplitReadStreamResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SplitReadStreamResponse.ProtoReflect.Descriptor instead.
func (*SplitReadStreamResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{6}
}

func (x *SplitReadStreamResponse) GetPrimaryStream() *ReadStream {
	if x != nil {
		return x.PrimaryStream
	}
	return nil
}

func (x *SplitReadStreamResponse) GetRemainderStream() *ReadStream {
	if x != nil {
		return x.RemainderStream
	}
	return nil
}

// Request message for `CreateWriteStream`.
type CreateWriteStreamRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Reference to the table to which the stream belongs, in the format
	// of `projects/{project}/datasets/{dataset}/tables/{table}`.
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Required. Stream to be created.
	WriteStream   *WriteStream `protobuf:"bytes,2,opt,name=write_stream,json=writeStream,proto3" json:"write_stream,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateWriteStreamRequest) Reset() {
	*x = CreateWriteStreamRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateWriteStreamRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateWriteStreamRequest) ProtoMessage() {}

func (x *CreateWriteStreamRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateWriteStreamRequest.ProtoReflect.Descriptor instead.
func (*CreateWriteStreamRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{7}
}

func (x *CreateWriteStreamRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *CreateWriteStreamRequest) GetWriteStream() *WriteStream {
	if x != nil {
		return x.WriteStream
	}
	return nil
}

// Request message for `AppendRows`.
//
// Because AppendRows is a bidirectional streaming RPC, certain parts of the
// AppendRowsRequest need only be specified for the first request before
// switching table destinations. You can also switch table destinations within
// the same connection for the default stream.
//
// The size of a single AppendRowsRequest must be less than 10 MB in size.
// Requests larger than this return an error, typically `INVALID_ARGUMENT`.
type AppendRowsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The write_stream identifies the append operation. It must be
	// provided in the following scenarios:
	//
	// * In the first request to an AppendRows connection.
	//
	// * In all subsequent requests to an AppendRows connection, if you use the
	// same connection to write to multiple tables or change the input schema for
	// default streams.
	//
	// For explicitly created write streams, the format is:
	//
	// * `projects/{project}/datasets/{dataset}/tables/{table}/streams/{id}`
	//
	// For the special default stream, the format is:
	//
	// * `projects/{project}/datasets/{dataset}/tables/{table}/streams/_default`.
	//
	// An example of a possible sequence of requests with write_stream fields
	// within a single connection:
	//
	// * r1: {write_stream: stream_name_1}
	//
	// * r2: {write_stream: /*omit*/}
	//
	// * r3: {write_stream: /*omit*/}
	//
	// * r4: {write_stream: stream_name_2}
	//
	// * r5: {write_stream: stream_name_2}
	//
	// The destination changed in request_4, so the write_stream field must be
	// populated in all subsequent requests in this stream.
	WriteStream string `protobuf:"bytes,1,opt,name=write_stream,json=writeStream,proto3" json:"write_stream,omitempty"`
	// If present, the write is only performed if the next append offset is same
	// as the provided value. If not present, the write is performed at the
	// current end of stream. Specifying a value for this field is not allowed
	// when calling AppendRows for the '_default' stream.
	Offset *wrapperspb.Int64Value `protobuf:"bytes,2,opt,name=offset,proto3" json:"offset,omitempty"`
	// Input rows. The `writer_schema` field must be specified at the initial
	// request and currently, it will be ignored if specified in following
	// requests. Following requests must have data in the same format as the
	// initial request.
	//
	// Types that are valid to be assigned to Rows:
	//
	//	*AppendRowsRequest_ProtoRows
	//	*AppendRowsRequest_ArrowRows
	Rows isAppendRowsRequest_Rows `protobuf_oneof:"rows"`
	// Id set by client to annotate its identity. Only initial request setting is
	// respected.
	TraceId string `protobuf:"bytes,6,opt,name=trace_id,json=traceId,proto3" json:"trace_id,omitempty"`
	// A map to indicate how to interpret missing value for some fields. Missing
	// values are fields present in user schema but missing in rows. The key is
	// the field name. The value is the interpretation of missing values for the
	// field.
	//
	// For example, a map {'foo': NULL_VALUE, 'bar': DEFAULT_VALUE} means all
	// missing values in field foo are interpreted as NULL, all missing values in
	// field bar are interpreted as the default value of field bar in table
	// schema.
	//
	// If a field is not in this map and has missing values, the missing values
	// in this field are interpreted as NULL.
	//
	// This field only applies to the current request, it won't affect other
	// requests on the connection.
	//
	// Currently, field name can only be top-level column name, can't be a struct
	// field path like 'foo.bar'.
	MissingValueInterpretations map[string]AppendRowsRequest_MissingValueInterpretation `protobuf:"bytes,7,rep,name=missing_value_interpretations,json=missingValueInterpretations,proto3" json:"missing_value_interpretations,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"varint,2,opt,name=value,enum=google.cloud.bigquery.storage.v1.AppendRowsRequest_MissingValueInterpretation"`
	// Optional. Default missing value interpretation for all columns in the
	// table. When a value is specified on an `AppendRowsRequest`, it is applied
	// to all requests from that point forward, until a subsequent
	// `AppendRowsRequest` sets it to a different value.
	// `missing_value_interpretation` can override
	// `default_missing_value_interpretation`. For example, if you want to write
	// `NULL` instead of using default values for some columns, you can set
	// `default_missing_value_interpretation` to `DEFAULT_VALUE` and at the same
	// time, set `missing_value_interpretations` to `NULL_VALUE` on those columns.
	DefaultMissingValueInterpretation AppendRowsRequest_MissingValueInterpretation `protobuf:"varint,8,opt,name=default_missing_value_interpretation,json=defaultMissingValueInterpretation,proto3,enum=google.cloud.bigquery.storage.v1.AppendRowsRequest_MissingValueInterpretation" json:"default_missing_value_interpretation,omitempty"`
	unknownFields                     protoimpl.UnknownFields
	sizeCache                         protoimpl.SizeCache
}

func (x *AppendRowsRequest) Reset() {
	*x = AppendRowsRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppendRowsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppendRowsRequest) ProtoMessage() {}

func (x *AppendRowsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppendRowsRequest.ProtoReflect.Descriptor instead.
func (*AppendRowsRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{8}
}

func (x *AppendRowsRequest) GetWriteStream() string {
	if x != nil {
		return x.WriteStream
	}
	return ""
}

func (x *AppendRowsRequest) GetOffset() *wrapperspb.Int64Value {
	if x != nil {
		return x.Offset
	}
	return nil
}

func (x *AppendRowsRequest) GetRows() isAppendRowsRequest_Rows {
	if x != nil {
		return x.Rows
	}
	return nil
}

func (x *AppendRowsRequest) GetProtoRows() *AppendRowsRequest_ProtoData {
	if x != nil {
		if x, ok := x.Rows.(*AppendRowsRequest_ProtoRows); ok {
			return x.ProtoRows
		}
	}
	return nil
}

func (x *AppendRowsRequest) GetArrowRows() *AppendRowsRequest_ArrowData {
	if x != nil {
		if x, ok := x.Rows.(*AppendRowsRequest_ArrowRows); ok {
			return x.ArrowRows
		}
	}
	return nil
}

func (x *AppendRowsRequest) GetTraceId() string {
	if x != nil {
		return x.TraceId
	}
	return ""
}

func (x *AppendRowsRequest) GetMissingValueInterpretations() map[string]AppendRowsRequest_MissingValueInterpretation {
	if x != nil {
		return x.MissingValueInterpretations
	}
	return nil
}

func (x *AppendRowsRequest) GetDefaultMissingValueInterpretation() AppendRowsRequest_MissingValueInterpretation {
	if x != nil {
		return x.DefaultMissingValueInterpretation
	}
	return AppendRowsRequest_MISSING_VALUE_INTERPRETATION_UNSPECIFIED
}

type isAppendRowsRequest_Rows interface {
	isAppendRowsRequest_Rows()
}

type AppendRowsRequest_ProtoRows struct {
	// Rows in proto format.
	ProtoRows *AppendRowsRequest_ProtoData `protobuf:"bytes,4,opt,name=proto_rows,json=protoRows,proto3,oneof"`
}

type AppendRowsRequest_ArrowRows struct {
	// Rows in arrow format.
	ArrowRows *AppendRowsRequest_ArrowData `protobuf:"bytes,5,opt,name=arrow_rows,json=arrowRows,proto3,oneof"`
}

func (*AppendRowsRequest_ProtoRows) isAppendRowsRequest_Rows() {}

func (*AppendRowsRequest_ArrowRows) isAppendRowsRequest_Rows() {}

// Response message for `AppendRows`.
type AppendRowsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Response:
	//
	//	*AppendRowsResponse_AppendResult_
	//	*AppendRowsResponse_Error
	Response isAppendRowsResponse_Response `protobuf_oneof:"response"`
	// If backend detects a schema update, pass it to user so that user can
	// use it to input new type of message. It will be empty when no schema
	// updates have occurred.
	UpdatedSchema *TableSchema `protobuf:"bytes,3,opt,name=updated_schema,json=updatedSchema,proto3" json:"updated_schema,omitempty"`
	// If a request failed due to corrupted rows, no rows in the batch will be
	// appended. The API will return row level error info, so that the caller can
	// remove the bad rows and retry the request.
	RowErrors []*RowError `protobuf:"bytes,4,rep,name=row_errors,json=rowErrors,proto3" json:"row_errors,omitempty"`
	// The target of the append operation. Matches the write_stream in the
	// corresponding request.
	WriteStream   string `protobuf:"bytes,5,opt,name=write_stream,json=writeStream,proto3" json:"write_stream,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AppendRowsResponse) Reset() {
	*x = AppendRowsResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppendRowsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppendRowsResponse) ProtoMessage() {}

func (x *AppendRowsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppendRowsResponse.ProtoReflect.Descriptor instead.
func (*AppendRowsResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{9}
}

func (x *AppendRowsResponse) GetResponse() isAppendRowsResponse_Response {
	if x != nil {
		return x.Response
	}
	return nil
}

func (x *AppendRowsResponse) GetAppendResult() *AppendRowsResponse_AppendResult {
	if x != nil {
		if x, ok := x.Response.(*AppendRowsResponse_AppendResult_); ok {
			return x.AppendResult
		}
	}
	return nil
}

func (x *AppendRowsResponse) GetError() *status.Status {
	if x != nil {
		if x, ok := x.Response.(*AppendRowsResponse_Error); ok {
			return x.Error
		}
	}
	return nil
}

func (x *AppendRowsResponse) GetUpdatedSchema() *TableSchema {
	if x != nil {
		return x.UpdatedSchema
	}
	return nil
}

func (x *AppendRowsResponse) GetRowErrors() []*RowError {
	if x != nil {
		return x.RowErrors
	}
	return nil
}

func (x *AppendRowsResponse) GetWriteStream() string {
	if x != nil {
		return x.WriteStream
	}
	return ""
}

type isAppendRowsResponse_Response interface {
	isAppendRowsResponse_Response()
}

type AppendRowsResponse_AppendResult_ struct {
	// Result if the append is successful.
	AppendResult *AppendRowsResponse_AppendResult `protobuf:"bytes,1,opt,name=append_result,json=appendResult,proto3,oneof"`
}

type AppendRowsResponse_Error struct {
	// Error returned when problems were encountered.  If present,
	// it indicates rows were not accepted into the system.
	// Users can retry or continue with other append requests within the
	// same connection.
	//
	// Additional information about error signalling:
	//
	// ALREADY_EXISTS: Happens when an append specified an offset, and the
	// backend already has received data at this offset.  Typically encountered
	// in retry scenarios, and can be ignored.
	//
	// OUT_OF_RANGE: Returned when the specified offset in the stream is beyond
	// the current end of the stream.
	//
	// INVALID_ARGUMENT: Indicates a malformed request or data.
	//
	// ABORTED: Request processing is aborted because of prior failures.  The
	// request can be retried if previous failure is addressed.
	//
	// INTERNAL: Indicates server side error(s) that can be retried.
	Error *status.Status `protobuf:"bytes,2,opt,name=error,proto3,oneof"`
}

func (*AppendRowsResponse_AppendResult_) isAppendRowsResponse_Response() {}

func (*AppendRowsResponse_Error) isAppendRowsResponse_Response() {}

// Request message for `GetWriteStreamRequest`.
type GetWriteStreamRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the stream to get, in the form of
	// `projects/{project}/datasets/{dataset}/tables/{table}/streams/{stream}`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Indicates whether to get full or partial view of the WriteStream. If
	// not set, view returned will be basic.
	View          WriteStreamView `protobuf:"varint,3,opt,name=view,proto3,enum=google.cloud.bigquery.storage.v1.WriteStreamView" json:"view,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetWriteStreamRequest) Reset() {
	*x = GetWriteStreamRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetWriteStreamRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetWriteStreamRequest) ProtoMessage() {}

func (x *GetWriteStreamRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetWriteStreamRequest.ProtoReflect.Descriptor instead.
func (*GetWriteStreamRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{10}
}

func (x *GetWriteStreamRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *GetWriteStreamRequest) GetView() WriteStreamView {
	if x != nil {
		return x.View
	}
	return WriteStreamView_WRITE_STREAM_VIEW_UNSPECIFIED
}

// Request message for `BatchCommitWriteStreams`.
type BatchCommitWriteStreamsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Parent table that all the streams should belong to, in the form
	// of `projects/{project}/datasets/{dataset}/tables/{table}`.
	Parent string `protobuf:"bytes,1,opt,name=parent,proto3" json:"parent,omitempty"`
	// Required. The group of streams that will be committed atomically.
	WriteStreams  []string `protobuf:"bytes,2,rep,name=write_streams,json=writeStreams,proto3" json:"write_streams,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BatchCommitWriteStreamsRequest) Reset() {
	*x = BatchCommitWriteStreamsRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BatchCommitWriteStreamsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BatchCommitWriteStreamsRequest) ProtoMessage() {}

func (x *BatchCommitWriteStreamsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BatchCommitWriteStreamsRequest.ProtoReflect.Descriptor instead.
func (*BatchCommitWriteStreamsRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{11}
}

func (x *BatchCommitWriteStreamsRequest) GetParent() string {
	if x != nil {
		return x.Parent
	}
	return ""
}

func (x *BatchCommitWriteStreamsRequest) GetWriteStreams() []string {
	if x != nil {
		return x.WriteStreams
	}
	return nil
}

// Response message for `BatchCommitWriteStreams`.
type BatchCommitWriteStreamsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The time at which streams were committed in microseconds granularity.
	// This field will only exist when there are no stream errors.
	// **Note** if this field is not set, it means the commit was not successful.
	CommitTime *timestamppb.Timestamp `protobuf:"bytes,1,opt,name=commit_time,json=commitTime,proto3" json:"commit_time,omitempty"`
	// Stream level error if commit failed. Only streams with error will be in
	// the list.
	// If empty, there is no error and all streams are committed successfully.
	// If non empty, certain streams have errors and ZERO stream is committed due
	// to atomicity guarantee.
	StreamErrors  []*StorageError `protobuf:"bytes,2,rep,name=stream_errors,json=streamErrors,proto3" json:"stream_errors,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BatchCommitWriteStreamsResponse) Reset() {
	*x = BatchCommitWriteStreamsResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BatchCommitWriteStreamsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BatchCommitWriteStreamsResponse) ProtoMessage() {}

func (x *BatchCommitWriteStreamsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BatchCommitWriteStreamsResponse.ProtoReflect.Descriptor instead.
func (*BatchCommitWriteStreamsResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{12}
}

func (x *BatchCommitWriteStreamsResponse) GetCommitTime() *timestamppb.Timestamp {
	if x != nil {
		return x.CommitTime
	}
	return nil
}

func (x *BatchCommitWriteStreamsResponse) GetStreamErrors() []*StorageError {
	if x != nil {
		return x.StreamErrors
	}
	return nil
}

// Request message for invoking `FinalizeWriteStream`.
type FinalizeWriteStreamRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. Name of the stream to finalize, in the form of
	// `projects/{project}/datasets/{dataset}/tables/{table}/streams/{stream}`.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FinalizeWriteStreamRequest) Reset() {
	*x = FinalizeWriteStreamRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FinalizeWriteStreamRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FinalizeWriteStreamRequest) ProtoMessage() {}

func (x *FinalizeWriteStreamRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FinalizeWriteStreamRequest.ProtoReflect.Descriptor instead.
func (*FinalizeWriteStreamRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{13}
}

func (x *FinalizeWriteStreamRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Response message for `FinalizeWriteStream`.
type FinalizeWriteStreamResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Number of rows in the finalized stream.
	RowCount      int64 `protobuf:"varint,1,opt,name=row_count,json=rowCount,proto3" json:"row_count,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FinalizeWriteStreamResponse) Reset() {
	*x = FinalizeWriteStreamResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FinalizeWriteStreamResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FinalizeWriteStreamResponse) ProtoMessage() {}

func (x *FinalizeWriteStreamResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FinalizeWriteStreamResponse.ProtoReflect.Descriptor instead.
func (*FinalizeWriteStreamResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{14}
}

func (x *FinalizeWriteStreamResponse) GetRowCount() int64 {
	if x != nil {
		return x.RowCount
	}
	return 0
}

// Request message for `FlushRows`.
type FlushRowsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The stream that is the target of the flush operation.
	WriteStream string `protobuf:"bytes,1,opt,name=write_stream,json=writeStream,proto3" json:"write_stream,omitempty"`
	// Ending offset of the flush operation. Rows before this offset(including
	// this offset) will be flushed.
	Offset        *wrapperspb.Int64Value `protobuf:"bytes,2,opt,name=offset,proto3" json:"offset,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FlushRowsRequest) Reset() {
	*x = FlushRowsRequest{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FlushRowsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FlushRowsRequest) ProtoMessage() {}

func (x *FlushRowsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FlushRowsRequest.ProtoReflect.Descriptor instead.
func (*FlushRowsRequest) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{15}
}

func (x *FlushRowsRequest) GetWriteStream() string {
	if x != nil {
		return x.WriteStream
	}
	return ""
}

func (x *FlushRowsRequest) GetOffset() *wrapperspb.Int64Value {
	if x != nil {
		return x.Offset
	}
	return nil
}

// Respond message for `FlushRows`.
type FlushRowsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The rows before this offset (including this offset) are flushed.
	Offset        int64 `protobuf:"varint,1,opt,name=offset,proto3" json:"offset,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FlushRowsResponse) Reset() {
	*x = FlushRowsResponse{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FlushRowsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FlushRowsResponse) ProtoMessage() {}

func (x *FlushRowsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FlushRowsResponse.ProtoReflect.Descriptor instead.
func (*FlushRowsResponse) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{16}
}

func (x *FlushRowsResponse) GetOffset() int64 {
	if x != nil {
		return x.Offset
	}
	return 0
}

// Structured custom BigQuery Storage error message. The error can be attached
// as error details in the returned rpc Status. In particular, the use of error
// codes allows more structured error handling, and reduces the need to evaluate
// unstructured error text strings.
type StorageError struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// BigQuery Storage specific error code.
	Code StorageError_StorageErrorCode `protobuf:"varint,1,opt,name=code,proto3,enum=google.cloud.bigquery.storage.v1.StorageError_StorageErrorCode" json:"code,omitempty"`
	// Name of the failed entity.
	Entity string `protobuf:"bytes,2,opt,name=entity,proto3" json:"entity,omitempty"`
	// Message that describes the error.
	ErrorMessage  string `protobuf:"bytes,3,opt,name=error_message,json=errorMessage,proto3" json:"error_message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StorageError) Reset() {
	*x = StorageError{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StorageError) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StorageError) ProtoMessage() {}

func (x *StorageError) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StorageError.ProtoReflect.Descriptor instead.
func (*StorageError) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{17}
}

func (x *StorageError) GetCode() StorageError_StorageErrorCode {
	if x != nil {
		return x.Code
	}
	return StorageError_STORAGE_ERROR_CODE_UNSPECIFIED
}

func (x *StorageError) GetEntity() string {
	if x != nil {
		return x.Entity
	}
	return ""
}

func (x *StorageError) GetErrorMessage() string {
	if x != nil {
		return x.ErrorMessage
	}
	return ""
}

// The message that presents row level error info in a request.
type RowError struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Index of the malformed row in the request.
	Index int64 `protobuf:"varint,1,opt,name=index,proto3" json:"index,omitempty"`
	// Structured error reason for a row error.
	Code RowError_RowErrorCode `protobuf:"varint,2,opt,name=code,proto3,enum=google.cloud.bigquery.storage.v1.RowError_RowErrorCode" json:"code,omitempty"`
	// Description of the issue encountered when processing the row.
	Message       string `protobuf:"bytes,3,opt,name=message,proto3" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RowError) Reset() {
	*x = RowError{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RowError) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RowError) ProtoMessage() {}

func (x *RowError) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RowError.ProtoReflect.Descriptor instead.
func (*RowError) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{18}
}

func (x *RowError) GetIndex() int64 {
	if x != nil {
		return x.Index
	}
	return 0
}

func (x *RowError) GetCode() RowError_RowErrorCode {
	if x != nil {
		return x.Code
	}
	return RowError_ROW_ERROR_CODE_UNSPECIFIED
}

func (x *RowError) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

type StreamStats_Progress struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The fraction of rows assigned to the stream that have been processed by
	// the server so far, not including the rows in the current response
	// message.
	//
	// This value, along with `at_response_end`, can be used to interpolate
	// the progress made as the rows in the message are being processed using
	// the following formula: `at_response_start + (at_response_end -
	// at_response_start) * rows_processed_from_response / rows_in_response`.
	//
	// Note that if a filter is provided, the `at_response_end` value of the
	// previous response may not necessarily be equal to the
	// `at_response_start` value of the current response.
	AtResponseStart float64 `protobuf:"fixed64,1,opt,name=at_response_start,json=atResponseStart,proto3" json:"at_response_start,omitempty"`
	// Similar to `at_response_start`, except that this value includes the
	// rows in the current response.
	AtResponseEnd float64 `protobuf:"fixed64,2,opt,name=at_response_end,json=atResponseEnd,proto3" json:"at_response_end,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamStats_Progress) Reset() {
	*x = StreamStats_Progress{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[19]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamStats_Progress) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamStats_Progress) ProtoMessage() {}

func (x *StreamStats_Progress) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[19]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamStats_Progress.ProtoReflect.Descriptor instead.
func (*StreamStats_Progress) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{3, 0}
}

func (x *StreamStats_Progress) GetAtResponseStart() float64 {
	if x != nil {
		return x.AtResponseStart
	}
	return 0
}

func (x *StreamStats_Progress) GetAtResponseEnd() float64 {
	if x != nil {
		return x.AtResponseEnd
	}
	return 0
}

// Arrow schema and data.
type AppendRowsRequest_ArrowData struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. Arrow Schema used to serialize the data.
	WriterSchema *ArrowSchema `protobuf:"bytes,1,opt,name=writer_schema,json=writerSchema,proto3" json:"writer_schema,omitempty"`
	// Required. Serialized row data in Arrow format.
	Rows          *ArrowRecordBatch `protobuf:"bytes,2,opt,name=rows,proto3" json:"rows,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AppendRowsRequest_ArrowData) Reset() {
	*x = AppendRowsRequest_ArrowData{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[20]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppendRowsRequest_ArrowData) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppendRowsRequest_ArrowData) ProtoMessage() {}

func (x *AppendRowsRequest_ArrowData) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[20]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppendRowsRequest_ArrowData.ProtoReflect.Descriptor instead.
func (*AppendRowsRequest_ArrowData) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{8, 0}
}

func (x *AppendRowsRequest_ArrowData) GetWriterSchema() *ArrowSchema {
	if x != nil {
		return x.WriterSchema
	}
	return nil
}

func (x *AppendRowsRequest_ArrowData) GetRows() *ArrowRecordBatch {
	if x != nil {
		return x.Rows
	}
	return nil
}

// ProtoData contains the data rows and schema when constructing append
// requests.
type AppendRowsRequest_ProtoData struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. The protocol buffer schema used to serialize the data. Provide
	// this value whenever:
	//
	// * You send the first request of an RPC connection.
	//
	// * You change the input schema.
	//
	// * You specify a new destination table.
	WriterSchema *ProtoSchema `protobuf:"bytes,1,opt,name=writer_schema,json=writerSchema,proto3" json:"writer_schema,omitempty"`
	// Required. Serialized row data in protobuf message format.
	// Currently, the backend expects the serialized rows to adhere to
	// proto2 semantics when appending rows, particularly with respect to
	// how default values are encoded.
	Rows          *ProtoRows `protobuf:"bytes,2,opt,name=rows,proto3" json:"rows,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AppendRowsRequest_ProtoData) Reset() {
	*x = AppendRowsRequest_ProtoData{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[21]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppendRowsRequest_ProtoData) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppendRowsRequest_ProtoData) ProtoMessage() {}

func (x *AppendRowsRequest_ProtoData) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[21]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppendRowsRequest_ProtoData.ProtoReflect.Descriptor instead.
func (*AppendRowsRequest_ProtoData) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{8, 1}
}

func (x *AppendRowsRequest_ProtoData) GetWriterSchema() *ProtoSchema {
	if x != nil {
		return x.WriterSchema
	}
	return nil
}

func (x *AppendRowsRequest_ProtoData) GetRows() *ProtoRows {
	if x != nil {
		return x.Rows
	}
	return nil
}

// AppendResult is returned for successful append requests.
type AppendRowsResponse_AppendResult struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The row offset at which the last append occurred. The offset will not be
	// set if appending using default streams.
	Offset        *wrapperspb.Int64Value `protobuf:"bytes,1,opt,name=offset,proto3" json:"offset,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AppendRowsResponse_AppendResult) Reset() {
	*x = AppendRowsResponse_AppendResult{}
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[23]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppendRowsResponse_AppendResult) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppendRowsResponse_AppendResult) ProtoMessage() {}

func (x *AppendRowsResponse_AppendResult) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[23]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppendRowsResponse_AppendResult.ProtoReflect.Descriptor instead.
func (*AppendRowsResponse_AppendResult) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP(), []int{9, 0}
}

func (x *AppendRowsResponse_AppendResult) GetOffset() *wrapperspb.Int64Value {
	if x != nil {
		return x.Offset
	}
	return nil
}

var File_google_cloud_bigquery_storage_v1_storage_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_storage_proto_rawDesc = "" +
	"\n" +
	".google/cloud/bigquery/storage/v1/storage.proto\x12 google.cloud.bigquery.storage.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a,google/cloud/bigquery/storage/v1/arrow.proto\x1a+google/cloud/bigquery/storage/v1/avro.proto\x1a/google/cloud/bigquery/storage/v1/protobuf.proto\x1a-google/cloud/bigquery/storage/v1/stream.proto\x1a,google/cloud/bigquery/storage/v1/table.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x17google/rpc/status.proto\"\xa5\x02\n" +
	"\x18CreateReadSessionRequest\x12K\n" +
	"\x06parent\x18\x01 \x01(\tB3\xe0A\x02\xfaA-\n" +
	"+cloudresourcemanager.googleapis.com/ProjectR\x06parent\x12U\n" +
	"\fread_session\x18\x02 \x01(\v2-.google.cloud.bigquery.storage.v1.ReadSessionB\x03\xe0A\x02R\vreadSession\x12(\n" +
	"\x10max_stream_count\x18\x03 \x01(\x05R\x0emaxStreamCount\x12;\n" +
	"\x1apreferred_min_stream_count\x18\x04 \x01(\x05R\x17preferredMinStreamCount\"}\n" +
	"\x0fReadRowsRequest\x12R\n" +
	"\vread_stream\x18\x01 \x01(\tB1\xe0A\x02\xfaA+\n" +
	")bigquerystorage.googleapis.com/ReadStreamR\n" +
	"readStream\x12\x16\n" +
	"\x06offset\x18\x02 \x01(\x03R\x06offset\":\n" +
	"\rThrottleState\x12)\n" +
	"\x10throttle_percent\x18\x01 \x01(\x05R\x0fthrottlePercent\"\xc1\x01\n" +
	"\vStreamStats\x12R\n" +
	"\bprogress\x18\x02 \x01(\v26.google.cloud.bigquery.storage.v1.StreamStats.ProgressR\bprogress\x1a^\n" +
	"\bProgress\x12*\n" +
	"\x11at_response_start\x18\x01 \x01(\x01R\x0fatResponseStart\x12&\n" +
	"\x0fat_response_end\x18\x02 \x01(\x01R\ratResponseEnd\"\x97\x05\n" +
	"\x10ReadRowsResponse\x12I\n" +
	"\tavro_rows\x18\x03 \x01(\v2*.google.cloud.bigquery.storage.v1.AvroRowsH\x00R\bavroRows\x12b\n" +
	"\x12arrow_record_batch\x18\x04 \x01(\v22.google.cloud.bigquery.storage.v1.ArrowRecordBatchH\x00R\x10arrowRecordBatch\x12\x1b\n" +
	"\trow_count\x18\x06 \x01(\x03R\browCount\x12C\n" +
	"\x05stats\x18\x02 \x01(\v2-.google.cloud.bigquery.storage.v1.StreamStatsR\x05stats\x12V\n" +
	"\x0ethrottle_state\x18\x05 \x01(\v2/.google.cloud.bigquery.storage.v1.ThrottleStateR\rthrottleState\x12T\n" +
	"\vavro_schema\x18\a \x01(\v2,.google.cloud.bigquery.storage.v1.AvroSchemaB\x03\xe0A\x03H\x01R\n" +
	"avroSchema\x12W\n" +
	"\farrow_schema\x18\b \x01(\v2-.google.cloud.bigquery.storage.v1.ArrowSchemaB\x03\xe0A\x03H\x01R\varrowSchema\x12>\n" +
	"\x16uncompressed_byte_size\x18\t \x01(\x03B\x03\xe0A\x01H\x02R\x14uncompressedByteSize\x88\x01\x01B\x06\n" +
	"\x04rowsB\b\n" +
	"\x06schemaB\x19\n" +
	"\x17_uncompressed_byte_size\"{\n" +
	"\x16SplitReadStreamRequest\x12E\n" +
	"\x04name\x18\x01 \x01(\tB1\xe0A\x02\xfaA+\n" +
	")bigquerystorage.googleapis.com/ReadStreamR\x04name\x12\x1a\n" +
	"\bfraction\x18\x02 \x01(\x01R\bfraction\"\xc7\x01\n" +
	"\x17SplitReadStreamResponse\x12S\n" +
	"\x0eprimary_stream\x18\x01 \x01(\v2,.google.cloud.bigquery.storage.v1.ReadStreamR\rprimaryStream\x12W\n" +
	"\x10remainder_stream\x18\x02 \x01(\v2,.google.cloud.bigquery.storage.v1.ReadStreamR\x0fremainderStream\"\xb0\x01\n" +
	"\x18CreateWriteStreamRequest\x12=\n" +
	"\x06parent\x18\x01 \x01(\tB%\xe0A\x02\xfaA\x1f\n" +
	"\x1dbigquery.googleapis.com/TableR\x06parent\x12U\n" +
	"\fwrite_stream\x18\x02 \x01(\v2-.google.cloud.bigquery.storage.v1.WriteStreamB\x03\xe0A\x02R\vwriteStream\"\xa1\n" +
	"\n" +
	"\x11AppendRowsRequest\x12U\n" +
	"\fwrite_stream\x18\x01 \x01(\tB2\xe0A\x02\xfaA,\n" +
	"*bigquerystorage.googleapis.com/WriteStreamR\vwriteStream\x123\n" +
	"\x06offset\x18\x02 \x01(\v2\x1b.google.protobuf.Int64ValueR\x06offset\x12^\n" +
	"\n" +
	"proto_rows\x18\x04 \x01(\v2=.google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoDataH\x00R\tprotoRows\x12^\n" +
	"\n" +
	"arrow_rows\x18\x05 \x01(\v2=.google.cloud.bigquery.storage.v1.AppendRowsRequest.ArrowDataH\x00R\tarrowRows\x12\x19\n" +
	"\btrace_id\x18\x06 \x01(\tR\atraceId\x12\x98\x01\n" +
	"\x1dmissing_value_interpretations\x18\a \x03(\v2T.google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationsEntryR\x1bmissingValueInterpretations\x12\xa4\x01\n" +
	"$default_missing_value_interpretation\x18\b \x01(\x0e2N.google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationB\x03\xe0A\x01R!defaultMissingValueInterpretation\x1a\xa7\x01\n" +
	"\tArrowData\x12R\n" +
	"\rwriter_schema\x18\x01 \x01(\v2-.google.cloud.bigquery.storage.v1.ArrowSchemaR\fwriterSchema\x12F\n" +
	"\x04rows\x18\x02 \x01(\v22.google.cloud.bigquery.storage.v1.ArrowRecordBatchR\x04rows\x1a\xa0\x01\n" +
	"\tProtoData\x12R\n" +
	"\rwriter_schema\x18\x01 \x01(\v2-.google.cloud.bigquery.storage.v1.ProtoSchemaR\fwriterSchema\x12?\n" +
	"\x04rows\x18\x02 \x01(\v2+.google.cloud.bigquery.storage.v1.ProtoRowsR\x04rows\x1a\x9e\x01\n" +
	" MissingValueInterpretationsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12d\n" +
	"\x05value\x18\x02 \x01(\x0e2N.google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationR\x05value:\x028\x01\"m\n" +
	"\x1aMissingValueInterpretation\x12,\n" +
	"(MISSING_VALUE_INTERPRETATION_UNSPECIFIED\x10\x00\x12\x0e\n" +
	"\n" +
	"NULL_VALUE\x10\x01\x12\x11\n" +
	"\rDEFAULT_VALUE\x10\x02B\x06\n" +
	"\x04rows\"\xbf\x03\n" +
	"\x12AppendRowsResponse\x12h\n" +
	"\rappend_result\x18\x01 \x01(\v2A.google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResultH\x00R\fappendResult\x12*\n" +
	"\x05error\x18\x02 \x01(\v2\x12.google.rpc.StatusH\x00R\x05error\x12T\n" +
	"\x0eupdated_schema\x18\x03 \x01(\v2-.google.cloud.bigquery.storage.v1.TableSchemaR\rupdatedSchema\x12I\n" +
	"\n" +
	"row_errors\x18\x04 \x03(\v2*.google.cloud.bigquery.storage.v1.RowErrorR\trowErrors\x12!\n" +
	"\fwrite_stream\x18\x05 \x01(\tR\vwriteStream\x1aC\n" +
	"\fAppendResult\x123\n" +
	"\x06offset\x18\x01 \x01(\v2\x1b.google.protobuf.Int64ValueR\x06offsetB\n" +
	"\n" +
	"\bresponse\"\xa6\x01\n" +
	"\x15GetWriteStreamRequest\x12F\n" +
	"\x04name\x18\x01 \x01(\tB2\xe0A\x02\xfaA,\n" +
	"*bigquerystorage.googleapis.com/WriteStreamR\x04name\x12E\n" +
	"\x04view\x18\x03 \x01(\x0e21.google.cloud.bigquery.storage.v1.WriteStreamViewR\x04view\"\x89\x01\n" +
	"\x1eBatchCommitWriteStreamsRequest\x12=\n" +
	"\x06parent\x18\x01 \x01(\tB%\xe0A\x02\xfaA\x1f\n" +
	"\x1dbigquery.googleapis.com/TableR\x06parent\x12(\n" +
	"\rwrite_streams\x18\x02 \x03(\tB\x03\xe0A\x02R\fwriteStreams\"\xb3\x01\n" +
	"\x1fBatchCommitWriteStreamsResponse\x12;\n" +
	"\vcommit_time\x18\x01 \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"commitTime\x12S\n" +
	"\rstream_errors\x18\x02 \x03(\v2..google.cloud.bigquery.storage.v1.StorageErrorR\fstreamErrors\"d\n" +
	"\x1aFinalizeWriteStreamRequest\x12F\n" +
	"\x04name\x18\x01 \x01(\tB2\xe0A\x02\xfaA,\n" +
	"*bigquerystorage.googleapis.com/WriteStreamR\x04name\":\n" +
	"\x1bFinalizeWriteStreamResponse\x12\x1b\n" +
	"\trow_count\x18\x01 \x01(\x03R\browCount\"\x9e\x01\n" +
	"\x10FlushRowsRequest\x12U\n" +
	"\fwrite_stream\x18\x01 \x01(\tB2\xe0A\x02\xfaA,\n" +
	"*bigquerystorage.googleapis.com/WriteStreamR\vwriteStream\x123\n" +
	"\x06offset\x18\x02 \x01(\v2\x1b.google.protobuf.Int64ValueR\x06offset\"+\n" +
	"\x11FlushRowsResponse\x12\x16\n" +
	"\x06offset\x18\x01 \x01(\x03R\x06offset\"\xc0\x04\n" +
	"\fStorageError\x12S\n" +
	"\x04code\x18\x01 \x01(\x0e2?.google.cloud.bigquery.storage.v1.StorageError.StorageErrorCodeR\x04code\x12\x16\n" +
	"\x06entity\x18\x02 \x01(\tR\x06entity\x12#\n" +
	"\rerror_message\x18\x03 \x01(\tR\ferrorMessage\"\x9d\x03\n" +
	"\x10StorageErrorCode\x12\"\n" +
	"\x1eSTORAGE_ERROR_CODE_UNSPECIFIED\x10\x00\x12\x13\n" +
	"\x0fTABLE_NOT_FOUND\x10\x01\x12\x1c\n" +
	"\x18STREAM_ALREADY_COMMITTED\x10\x02\x12\x14\n" +
	"\x10STREAM_NOT_FOUND\x10\x03\x12\x17\n" +
	"\x13INVALID_STREAM_TYPE\x10\x04\x12\x18\n" +
	"\x14INVALID_STREAM_STATE\x10\x05\x12\x14\n" +
	"\x10STREAM_FINALIZED\x10\x06\x12 \n" +
	"\x1cSCHEMA_MISMATCH_EXTRA_FIELDS\x10\a\x12\x19\n" +
	"\x15OFFSET_ALREADY_EXISTS\x10\b\x12\x17\n" +
	"\x13OFFSET_OUT_OF_RANGE\x10\t\x12\x15\n" +
	"\x11CMEK_NOT_PROVIDED\x10\n" +
	"\x12\x19\n" +
	"\x15INVALID_CMEK_PROVIDED\x10\v\x12\x19\n" +
	"\x15CMEK_ENCRYPTION_ERROR\x10\f\x12\x15\n" +
	"\x11KMS_SERVICE_ERROR\x10\r\x12\x19\n" +
	"\x15KMS_PERMISSION_DENIED\x10\x0e\"\xc9\x01\n" +
	"\bRowError\x12\x14\n" +
	"\x05index\x18\x01 \x01(\x03R\x05index\x12K\n" +
	"\x04code\x18\x02 \x01(\x0e27.google.cloud.bigquery.storage.v1.RowError.RowErrorCodeR\x04code\x12\x18\n" +
	"\amessage\x18\x03 \x01(\tR\amessage\"@\n" +
	"\fRowErrorCode\x12\x1e\n" +
	"\x1aROW_ERROR_CODE_UNSPECIFIED\x10\x00\x12\x10\n" +
	"\fFIELDS_ERROR\x10\x012\x92\x06\n" +
	"\fBigQueryRead\x12\xe9\x01\n" +
	"\x11CreateReadSession\x12:.google.cloud.bigquery.storage.v1.CreateReadSessionRequest\x1a-.google.cloud.bigquery.storage.v1.ReadSession\"i\xdaA$parent,read_session,max_stream_count\x82\xd3\xe4\x93\x02<:\x01*\"7/v1/{read_session.table=projects/*/datasets/*/tables/*}\x12\xcf\x01\n" +
	"\bReadRows\x121.google.cloud.bigquery.storage.v1.ReadRowsRequest\x1a2.google.cloud.bigquery.storage.v1.ReadRowsResponse\"Z\xdaA\x12read_stream,offset\x82\xd3\xe4\x93\x02?\x12=/v1/{read_stream=projects/*/locations/*/sessions/*/streams/*}0\x01\x12\xc6\x01\n" +
	"\x0fSplitReadStream\x128.google.cloud.bigquery.storage.v1.SplitReadStreamRequest\x1a9.google.cloud.bigquery.storage.v1.SplitReadStreamResponse\">\x82\xd3\xe4\x93\x028\x126/v1/{name=projects/*/locations/*/sessions/*/streams/*}\x1a{\xcaA\x1ebigquerystorage.googleapis.com\xd2AWhttps://www.googleapis.com/auth/bigquery,https://www.googleapis.com/auth/cloud-platform2\xbc\v\n" +
	"\rBigQueryWrite\x12\xd7\x01\n" +
	"\x11CreateWriteStream\x12:.google.cloud.bigquery.storage.v1.CreateWriteStreamRequest\x1a-.google.cloud.bigquery.storage.v1.WriteStream\"W\xdaA\x13parent,write_stream\x82\xd3\xe4\x93\x02;:\fwrite_stream\"+/v1/{parent=projects/*/datasets/*/tables/*}\x12\xd2\x01\n" +
	"\n" +
	"AppendRows\x123.google.cloud.bigquery.storage.v1.AppendRowsRequest\x1a4.google.cloud.bigquery.storage.v1.AppendRowsResponse\"U\xdaA\fwrite_stream\x82\xd3\xe4\x93\x02@:\x01*\";/v1/{write_stream=projects/*/datasets/*/tables/*/streams/*}(\x010\x01\x12\xbf\x01\n" +
	"\x0eGetWriteStream\x127.google.cloud.bigquery.storage.v1.GetWriteStreamRequest\x1a-.google.cloud.bigquery.storage.v1.WriteStream\"E\xdaA\x04name\x82\xd3\xe4\x93\x028:\x01*\"3/v1/{name=projects/*/datasets/*/tables/*/streams/*}\x12\xd9\x01\n" +
	"\x13FinalizeWriteStream\x12<.google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest\x1a=.google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse\"E\xdaA\x04name\x82\xd3\xe4\x93\x028:\x01*\"3/v1/{name=projects/*/datasets/*/tables/*/streams/*}\x12\xdc\x01\n" +
	"\x17BatchCommitWriteStreams\x12@.google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest\x1aA.google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse\"<\xdaA\x06parent\x82\xd3\xe4\x93\x02-\x12+/v1/{parent=projects/*/datasets/*/tables/*}\x12\xcb\x01\n" +
	"\tFlushRows\x122.google.cloud.bigquery.storage.v1.FlushRowsRequest\x1a3.google.cloud.bigquery.storage.v1.FlushRowsResponse\"U\xdaA\fwrite_stream\x82\xd3\xe4\x93\x02@:\x01*\";/v1/{write_stream=projects/*/datasets/*/tables/*/streams/*}\x1a\xb0\x01\xcaA\x1ebigquerystorage.googleapis.com\xd2A\x8b\x01https://www.googleapis.com/auth/bigquery,https://www.googleapis.com/auth/bigquery.insertdata,https://www.googleapis.com/auth/cloud-platformB\x94\x02\xeaAU\n" +
	"\x1dbigquery.googleapis.com/Table\x124projects/{project}/datasets/{dataset}/tables/{table}\n" +
	"$com.google.cloud.bigquery.storage.v1B\fStorageProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var (
	file_google_cloud_bigquery_storage_v1_storage_proto_rawDescOnce sync.Once
	file_google_cloud_bigquery_storage_v1_storage_proto_rawDescData []byte
)

func file_google_cloud_bigquery_storage_v1_storage_proto_rawDescGZIP() []byte {
	file_google_cloud_bigquery_storage_v1_storage_proto_rawDescOnce.Do(func() {
		file_google_cloud_bigquery_storage_v1_storage_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_storage_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_storage_proto_rawDesc)))
	})
	return file_google_cloud_bigquery_storage_v1_storage_proto_rawDescData
}

var file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes = make([]protoimpl.EnumInfo, 3)
var file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes = make([]protoimpl.MessageInfo, 24)
var file_google_cloud_bigquery_storage_v1_storage_proto_goTypes = []any{
	(AppendRowsRequest_MissingValueInterpretation)(0), // 0: google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation
	(StorageError_StorageErrorCode)(0),                // 1: google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode
	(RowError_RowErrorCode)(0),                        // 2: google.cloud.bigquery.storage.v1.RowError.RowErrorCode
	(*CreateReadSessionRequest)(nil),                  // 3: google.cloud.bigquery.storage.v1.CreateReadSessionRequest
	(*ReadRowsRequest)(nil),                           // 4: google.cloud.bigquery.storage.v1.ReadRowsRequest
	(*ThrottleState)(nil),                             // 5: google.cloud.bigquery.storage.v1.ThrottleState
	(*StreamStats)(nil),                               // 6: google.cloud.bigquery.storage.v1.StreamStats
	(*ReadRowsResponse)(nil),                          // 7: google.cloud.bigquery.storage.v1.ReadRowsResponse
	(*SplitReadStreamRequest)(nil),                    // 8: google.cloud.bigquery.storage.v1.SplitReadStreamRequest
	(*SplitReadStreamResponse)(nil),                   // 9: google.cloud.bigquery.storage.v1.SplitReadStreamResponse
	(*CreateWriteStreamRequest)(nil),                  // 10: google.cloud.bigquery.storage.v1.CreateWriteStreamRequest
	(*AppendRowsRequest)(nil),                         // 11: google.cloud.bigquery.storage.v1.AppendRowsRequest
	(*AppendRowsResponse)(nil),                        // 12: google.cloud.bigquery.storage.v1.AppendRowsResponse
	(*GetWriteStreamRequest)(nil),                     // 13: google.cloud.bigquery.storage.v1.GetWriteStreamRequest
	(*BatchCommitWriteStreamsRequest)(nil),            // 14: google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest
	(*BatchCommitWriteStreamsResponse)(nil),           // 15: google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse
	(*FinalizeWriteStreamRequest)(nil),                // 16: google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest
	(*FinalizeWriteStreamResponse)(nil),               // 17: google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse
	(*FlushRowsRequest)(nil),                          // 18: google.cloud.bigquery.storage.v1.FlushRowsRequest
	(*FlushRowsResponse)(nil),                         // 19: google.cloud.bigquery.storage.v1.FlushRowsResponse
	(*StorageError)(nil),                              // 20: google.cloud.bigquery.storage.v1.StorageError
	(*RowError)(nil),                                  // 21: google.cloud.bigquery.storage.v1.RowError
	(*StreamStats_Progress)(nil),                      // 22: google.cloud.bigquery.storage.v1.StreamStats.Progress
	(*AppendRowsRequest_ArrowData)(nil),               // 23: google.cloud.bigquery.storage.v1.AppendRowsRequest.ArrowData
	(*AppendRowsRequest_ProtoData)(nil),               // 24: google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData
	nil,                                               // 25: google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationsEntry
	(*AppendRowsResponse_AppendResult)(nil),           // 26: google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult
	(*ReadSession)(nil),                               // 27: google.cloud.bigquery.storage.v1.ReadSession
	(*AvroRows)(nil),                                  // 28: google.cloud.bigquery.storage.v1.AvroRows
	(*ArrowRecordBatch)(nil),                          // 29: google.cloud.bigquery.storage.v1.ArrowRecordBatch
	(*AvroSchema)(nil),                                // 30: google.cloud.bigquery.storage.v1.AvroSchema
	(*ArrowSchema)(nil),                               // 31: google.cloud.bigquery.storage.v1.ArrowSchema
	(*ReadStream)(nil),                                // 32: google.cloud.bigquery.storage.v1.ReadStream
	(*WriteStream)(nil),                               // 33: google.cloud.bigquery.storage.v1.WriteStream
	(*wrapperspb.Int64Value)(nil),                     // 34: google.protobuf.Int64Value
	(*status.Status)(nil),                             // 35: google.rpc.Status
	(*TableSchema)(nil),                               // 36: google.cloud.bigquery.storage.v1.TableSchema
	(WriteStreamView)(0),                              // 37: google.cloud.bigquery.storage.v1.WriteStreamView
	(*timestamppb.Timestamp)(nil),                     // 38: google.protobuf.Timestamp
	(*ProtoSchema)(nil),                               // 39: google.cloud.bigquery.storage.v1.ProtoSchema
	(*ProtoRows)(nil),                                 // 40: google.cloud.bigquery.storage.v1.ProtoRows
}
var file_google_cloud_bigquery_storage_v1_storage_proto_depIdxs = []int32{
	27, // 0: google.cloud.bigquery.storage.v1.CreateReadSessionRequest.read_session:type_name -> google.cloud.bigquery.storage.v1.ReadSession
	22, // 1: google.cloud.bigquery.storage.v1.StreamStats.progress:type_name -> google.cloud.bigquery.storage.v1.StreamStats.Progress
	28, // 2: google.cloud.bigquery.storage.v1.ReadRowsResponse.avro_rows:type_name -> google.cloud.bigquery.storage.v1.AvroRows
	29, // 3: google.cloud.bigquery.storage.v1.ReadRowsResponse.arrow_record_batch:type_name -> google.cloud.bigquery.storage.v1.ArrowRecordBatch
	6,  // 4: google.cloud.bigquery.storage.v1.ReadRowsResponse.stats:type_name -> google.cloud.bigquery.storage.v1.StreamStats
	5,  // 5: google.cloud.bigquery.storage.v1.ReadRowsResponse.throttle_state:type_name -> google.cloud.bigquery.storage.v1.ThrottleState
	30, // 6: google.cloud.bigquery.storage.v1.ReadRowsResponse.avro_schema:type_name -> google.cloud.bigquery.storage.v1.AvroSchema
	31, // 7: google.cloud.bigquery.storage.v1.ReadRowsResponse.arrow_schema:type_name -> google.cloud.bigquery.storage.v1.ArrowSchema
	32, // 8: google.cloud.bigquery.storage.v1.SplitReadStreamResponse.primary_stream:type_name -> google.cloud.bigquery.storage.v1.ReadStream
	32, // 9: google.cloud.bigquery.storage.v1.SplitReadStreamResponse.remainder_stream:type_name -> google.cloud.bigquery.storage.v1.ReadStream
	33, // 10: google.cloud.bigquery.storage.v1.CreateWriteStreamRequest.write_stream:type_name -> google.cloud.bigquery.storage.v1.WriteStream
	34, // 11: google.cloud.bigquery.storage.v1.AppendRowsRequest.offset:type_name -> google.protobuf.Int64Value
	24, // 12: google.cloud.bigquery.storage.v1.AppendRowsRequest.proto_rows:type_name -> google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData
	23, // 13: google.cloud.bigquery.storage.v1.AppendRowsRequest.arrow_rows:type_name -> google.cloud.bigquery.storage.v1.AppendRowsRequest.ArrowData
	25, // 14: google.cloud.bigquery.storage.v1.AppendRowsRequest.missing_value_interpretations:type_name -> google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationsEntry
	0,  // 15: google.cloud.bigquery.storage.v1.AppendRowsRequest.default_missing_value_interpretation:type_name -> google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation
	26, // 16: google.cloud.bigquery.storage.v1.AppendRowsResponse.append_result:type_name -> google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult
	35, // 17: google.cloud.bigquery.storage.v1.AppendRowsResponse.error:type_name -> google.rpc.Status
	36, // 18: google.cloud.bigquery.storage.v1.AppendRowsResponse.updated_schema:type_name -> google.cloud.bigquery.storage.v1.TableSchema
	21, // 19: google.cloud.bigquery.storage.v1.AppendRowsResponse.row_errors:type_name -> google.cloud.bigquery.storage.v1.RowError
	37, // 20: google.cloud.bigquery.storage.v1.GetWriteStreamRequest.view:type_name -> google.cloud.bigquery.storage.v1.WriteStreamView
	38, // 21: google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse.commit_time:type_name -> google.protobuf.Timestamp
	20, // 22: google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse.stream_errors:type_name -> google.cloud.bigquery.storage.v1.StorageError
	34, // 23: google.cloud.bigquery.storage.v1.FlushRowsRequest.offset:type_name -> google.protobuf.Int64Value
	1,  // 24: google.cloud.bigquery.storage.v1.StorageError.code:type_name -> google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode
	2,  // 25: google.cloud.bigquery.storage.v1.RowError.code:type_name -> google.cloud.bigquery.storage.v1.RowError.RowErrorCode
	31, // 26: google.cloud.bigquery.storage.v1.AppendRowsRequest.ArrowData.writer_schema:type_name -> google.cloud.bigquery.storage.v1.ArrowSchema
	29, // 27: google.cloud.bigquery.storage.v1.AppendRowsRequest.ArrowData.rows:type_name -> google.cloud.bigquery.storage.v1.ArrowRecordBatch
	39, // 28: google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData.writer_schema:type_name -> google.cloud.bigquery.storage.v1.ProtoSchema
	40, // 29: google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData.rows:type_name -> google.cloud.bigquery.storage.v1.ProtoRows
	0,  // 30: google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationsEntry.value:type_name -> google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation
	34, // 31: google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult.offset:type_name -> google.protobuf.Int64Value
	3,  // 32: google.cloud.bigquery.storage.v1.BigQueryRead.CreateReadSession:input_type -> google.cloud.bigquery.storage.v1.CreateReadSessionRequest
	4,  // 33: google.cloud.bigquery.storage.v1.BigQueryRead.ReadRows:input_type -> google.cloud.bigquery.storage.v1.ReadRowsRequest
	8,  // 34: google.cloud.bigquery.storage.v1.BigQueryRead.SplitReadStream:input_type -> google.cloud.bigquery.storage.v1.SplitReadStreamRequest
	10, // 35: google.cloud.bigquery.storage.v1.BigQueryWrite.CreateWriteStream:input_type -> google.cloud.bigquery.storage.v1.CreateWriteStreamRequest
	11, // 36: google.cloud.bigquery.storage.v1.BigQueryWrite.AppendRows:input_type -> google.cloud.bigquery.storage.v1.AppendRowsRequest
	13, // 37: google.cloud.bigquery.storage.v1.BigQueryWrite.GetWriteStream:input_type -> google.cloud.bigquery.storage.v1.GetWriteStreamRequest
	16, // 38: google.cloud.bigquery.storage.v1.BigQueryWrite.FinalizeWriteStream:input_type -> google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest
	14, // 39: google.cloud.bigquery.storage.v1.BigQueryWrite.BatchCommitWriteStreams:input_type -> google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest
	18, // 40: google.cloud.bigquery.storage.v1.BigQueryWrite.FlushRows:input_type -> google.cloud.bigquery.storage.v1.FlushRowsRequest
	27, // 41: google.cloud.bigquery.storage.v1.BigQueryRead.CreateReadSession:output_type -> google.cloud.bigquery.storage.v1.ReadSession
	7,  // 42: google.cloud.bigquery.storage.v1.BigQueryRead.ReadRows:output_type -> google.cloud.bigquery.storage.v1.ReadRowsResponse
	9,  // 43: google.cloud.bigquery.storage.v1.BigQueryRead.SplitReadStream:output_type -> google.cloud.bigquery.storage.v1.SplitReadStreamResponse
	33, // 44: google.cloud.bigquery.storage.v1.BigQueryWrite.CreateWriteStream:output_type -> google.cloud.bigquery.storage.v1.WriteStream
	12, // 45: google.cloud.bigquery.storage.v1.BigQueryWrite.AppendRows:output_type -> google.cloud.bigquery.storage.v1.AppendRowsResponse
	33, // 46: google.cloud.bigquery.storage.v1.BigQueryWrite.GetWriteStream:output_type -> google.cloud.bigquery.storage.v1.WriteStream
	17, // 47: google.cloud.bigquery.storage.v1.BigQueryWrite.FinalizeWriteStream:output_type -> google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse
	15, // 48: google.cloud.bigquery.storage.v1.BigQueryWrite.BatchCommitWriteStreams:output_type -> google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse
	19, // 49: google.cloud.bigquery.storage.v1.BigQueryWrite.FlushRows:output_type -> google.cloud.bigquery.storage.v1.FlushRowsResponse
	41, // [41:50] is the sub-list for method output_type
	32, // [32:41] is the sub-list for method input_type
	32, // [32:32] is the sub-list for extension type_name
	32, // [32:32] is the sub-list for extension extendee
	0,  // [0:32] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_storage_proto_init() }
func file_google_cloud_bigquery_storage_v1_storage_proto_init() {
	if File_google_cloud_bigquery_storage_v1_storage_proto != nil {
		return
	}
	file_google_cloud_bigquery_storage_v1_arrow_proto_init()
	file_google_cloud_bigquery_storage_v1_avro_proto_init()
	file_google_cloud_bigquery_storage_v1_protobuf_proto_init()
	file_google_cloud_bigquery_storage_v1_stream_proto_init()
	file_google_cloud_bigquery_storage_v1_table_proto_init()
	file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[4].OneofWrappers = []any{
		(*ReadRowsResponse_AvroRows)(nil),
		(*ReadRowsResponse_ArrowRecordBatch)(nil),
		(*ReadRowsResponse_AvroSchema)(nil),
		(*ReadRowsResponse_ArrowSchema)(nil),
	}
	file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[8].OneofWrappers = []any{
		(*AppendRowsRequest_ProtoRows)(nil),
		(*AppendRowsRequest_ArrowRows)(nil),
	}
	file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes[9].OneofWrappers = []any{
		(*AppendRowsResponse_AppendResult_)(nil),
		(*AppendRowsResponse_Error)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_storage_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_storage_proto_rawDesc)),
			NumEnums:      3,
			NumMessages:   24,
			NumExtensions: 0,
			NumServices:   2,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_storage_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_storage_proto_depIdxs,
		EnumInfos:         file_google_cloud_bigquery_storage_v1_storage_proto_enumTypes,
		MessageInfos:      file_google_cloud_bigquery_storage_v1_storage_proto_msgTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_storage_proto = out.File
	file_google_cloud_bigquery_storage_v1_storage_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_storage_proto_depIdxs = nil
}
