// Code generated by ./cmd/ch-gen-col, DO NOT EDIT.

package proto

// ColUInt8 represents UInt8 column.
type ColUInt8 []uint8

// Compile-time assertions for ColUInt8.
var (
	_ ColInput  = ColUInt8{}
	_ ColResult = (*ColUInt8)(nil)
	_ Column    = (*ColUInt8)(nil)
)

// Rows returns count of rows in column.
func (c ColUInt8) Rows() int {
	return len(c)
}

// Reset resets data in row, preserving capacity for efficiency.
func (c *ColUInt8) Reset() {
	*c = (*c)[:0]
}

// Type returns ColumnType of UInt8.
func (ColUInt8) Type() ColumnType {
	return ColumnTypeUInt8
}

// Row returns i-th row of column.
func (c ColUInt8) Row(i int) uint8 {
	return c[i]
}

// Append uint8 to column.
func (c *ColUInt8) Append(v uint8) {
	*c = append(*c, v)
}

// Append uint8 slice to column.
func (c *ColUInt8) AppendArr(vs []uint8) {
	*c = append(*c, vs...)
}

// LowCardinality returns LowCardinality for UInt8.
func (c *ColUInt8) LowCardinality() *ColLowCardinality[uint8] {
	return &ColLowCardinality[uint8]{
		index: c,
	}
}

// Array is helper that creates Array of uint8.
func (c *ColUInt8) Array() *ColArr[uint8] {
	return &ColArr[uint8]{
		Data: c,
	}
}

// Nullable is helper that creates Nullable(uint8).
func (c *ColUInt8) Nullable() *ColNullable[uint8] {
	return &ColNullable[uint8]{
		Values: c,
	}
}

// NewArrUInt8 returns new Array(UInt8).
func NewArrUInt8() *ColArr[uint8] {
	return &ColArr[uint8]{
		Data: new(ColUInt8),
	}
}
