// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/annotations.proto

package storagepb

import (
	reflect "reflect"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var file_google_cloud_bigquery_storage_v1_annotations_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FieldOptions)(nil),
		ExtensionType: (*string)(nil),
		Field:         454943157,
		Name:          "google.cloud.bigquery.storage.v1.column_name",
		Tag:           "bytes,454943157,opt,name=column_name",
		Filename:      "google/cloud/bigquery/storage/v1/annotations.proto",
	},
}

// Extension fields to descriptorpb.FieldOptions.
var (
	// Setting the column_name extension allows users to reference
	// bigquery column independently of the field name in the protocol buffer
	// message.
	//
	// The intended use of this annotation is to reference a destination column
	// named using characters unavailable for protobuf field names (e.g. unicode
	// characters).
	//
	// More details about BigQuery naming limitations can be found here:
	// https://cloud.google.com/bigquery/docs/schemas#column_names
	//
	// This extension is currently experimental.
	//
	// optional string column_name = 454943157;
	E_ColumnName = &file_google_cloud_bigquery_storage_v1_annotations_proto_extTypes[0]
)

var File_google_cloud_bigquery_storage_v1_annotations_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_annotations_proto_rawDesc = "" +
	"\n" +
	"2google/cloud/bigquery/storage/v1/annotations.proto\x12 google.cloud.bigquery.storage.v1\x1a google/protobuf/descriptor.proto:E\n" +
	"\vcolumn_name\x12\x1d.google.protobuf.FieldOptions\x18\xb5\xc3\xf7\xd8\x01 \x01(\tR\n" +
	"columnName\x88\x01\x01B\xc0\x01\n" +
	"$com.google.cloud.bigquery.storage.v1B\x10AnnotationsProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var file_google_cloud_bigquery_storage_v1_annotations_proto_goTypes = []any{
	(*descriptorpb.FieldOptions)(nil), // 0: google.protobuf.FieldOptions
}
var file_google_cloud_bigquery_storage_v1_annotations_proto_depIdxs = []int32{
	0, // 0: google.cloud.bigquery.storage.v1.column_name:extendee -> google.protobuf.FieldOptions
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	0, // [0:1] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_annotations_proto_init() }
func file_google_cloud_bigquery_storage_v1_annotations_proto_init() {
	if File_google_cloud_bigquery_storage_v1_annotations_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_annotations_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_annotations_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 1,
			NumServices:   0,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_annotations_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_annotations_proto_depIdxs,
		ExtensionInfos:    file_google_cloud_bigquery_storage_v1_annotations_proto_extTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_annotations_proto = out.File
	file_google_cloud_bigquery_storage_v1_annotations_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_annotations_proto_depIdxs = nil
}
