// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/stream.proto

package storagepb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Data format for input or output data.
type DataFormat int32

const (
	// Data format is unspecified.
	DataFormat_DATA_FORMAT_UNSPECIFIED DataFormat = 0
	// Avro is a standard open source row based file format.
	// See https://avro.apache.org/ for more details.
	DataFormat_AVRO DataFormat = 1
	// Arrow is a standard open source column-based message format.
	// See https://arrow.apache.org/ for more details.
	DataFormat_ARROW DataFormat = 2
)

// Enum value maps for DataFormat.
var (
	DataFormat_name = map[int32]string{
		0: "DATA_FORMAT_UNSPECIFIED",
		1: "AVRO",
		2: "ARROW",
	}
	DataFormat_value = map[string]int32{
		"DATA_FORMAT_UNSPECIFIED": 0,
		"AVRO":                    1,
		"ARROW":                   2,
	}
)

func (x DataFormat) Enum() *DataFormat {
	p := new(DataFormat)
	*p = x
	return p
}

func (x DataFormat) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (DataFormat) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[0].Descriptor()
}

func (DataFormat) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[0]
}

func (x DataFormat) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use DataFormat.Descriptor instead.
func (DataFormat) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{0}
}

// WriteStreamView is a view enum that controls what details about a write
// stream should be returned.
type WriteStreamView int32

const (
	// The default / unset value.
	WriteStreamView_WRITE_STREAM_VIEW_UNSPECIFIED WriteStreamView = 0
	// The BASIC projection returns basic metadata about a write stream.  The
	// basic view does not include schema information.  This is the default view
	// returned by GetWriteStream.
	WriteStreamView_BASIC WriteStreamView = 1
	// The FULL projection returns all available write stream metadata, including
	// the schema.  CreateWriteStream returns the full projection of write stream
	// metadata.
	WriteStreamView_FULL WriteStreamView = 2
)

// Enum value maps for WriteStreamView.
var (
	WriteStreamView_name = map[int32]string{
		0: "WRITE_STREAM_VIEW_UNSPECIFIED",
		1: "BASIC",
		2: "FULL",
	}
	WriteStreamView_value = map[string]int32{
		"WRITE_STREAM_VIEW_UNSPECIFIED": 0,
		"BASIC":                         1,
		"FULL":                          2,
	}
)

func (x WriteStreamView) Enum() *WriteStreamView {
	p := new(WriteStreamView)
	*p = x
	return p
}

func (x WriteStreamView) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (WriteStreamView) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[1].Descriptor()
}

func (WriteStreamView) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[1]
}

func (x WriteStreamView) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use WriteStreamView.Descriptor instead.
func (WriteStreamView) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{1}
}

// Specifies which compression codec to attempt on the entire serialized
// response payload (either Arrow record batch or Avro rows). This is
// not to be confused with the Apache Arrow native compression codecs
// specified in ArrowSerializationOptions. For performance reasons, when
// creating a read session requesting Arrow responses, setting both native
// Arrow compression and application-level response compression will not be
// allowed - choose, at most, one kind of compression.
type ReadSession_TableReadOptions_ResponseCompressionCodec int32

const (
	// Default is no compression.
	ReadSession_TableReadOptions_RESPONSE_COMPRESSION_CODEC_UNSPECIFIED ReadSession_TableReadOptions_ResponseCompressionCodec = 0
	// Use raw LZ4 compression.
	ReadSession_TableReadOptions_RESPONSE_COMPRESSION_CODEC_LZ4 ReadSession_TableReadOptions_ResponseCompressionCodec = 2
)

// Enum value maps for ReadSession_TableReadOptions_ResponseCompressionCodec.
var (
	ReadSession_TableReadOptions_ResponseCompressionCodec_name = map[int32]string{
		0: "RESPONSE_COMPRESSION_CODEC_UNSPECIFIED",
		2: "RESPONSE_COMPRESSION_CODEC_LZ4",
	}
	ReadSession_TableReadOptions_ResponseCompressionCodec_value = map[string]int32{
		"RESPONSE_COMPRESSION_CODEC_UNSPECIFIED": 0,
		"RESPONSE_COMPRESSION_CODEC_LZ4":         2,
	}
)

func (x ReadSession_TableReadOptions_ResponseCompressionCodec) Enum() *ReadSession_TableReadOptions_ResponseCompressionCodec {
	p := new(ReadSession_TableReadOptions_ResponseCompressionCodec)
	*p = x
	return p
}

func (x ReadSession_TableReadOptions_ResponseCompressionCodec) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ReadSession_TableReadOptions_ResponseCompressionCodec) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[2].Descriptor()
}

func (ReadSession_TableReadOptions_ResponseCompressionCodec) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[2]
}

func (x ReadSession_TableReadOptions_ResponseCompressionCodec) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ReadSession_TableReadOptions_ResponseCompressionCodec.Descriptor instead.
func (ReadSession_TableReadOptions_ResponseCompressionCodec) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{0, 1, 0}
}

// Type enum of the stream.
type WriteStream_Type int32

const (
	// Unknown type.
	WriteStream_TYPE_UNSPECIFIED WriteStream_Type = 0
	// Data will commit automatically and appear as soon as the write is
	// acknowledged.
	WriteStream_COMMITTED WriteStream_Type = 1
	// Data is invisible until the stream is committed.
	WriteStream_PENDING WriteStream_Type = 2
	// Data is only visible up to the offset to which it was flushed.
	WriteStream_BUFFERED WriteStream_Type = 3
)

// Enum value maps for WriteStream_Type.
var (
	WriteStream_Type_name = map[int32]string{
		0: "TYPE_UNSPECIFIED",
		1: "COMMITTED",
		2: "PENDING",
		3: "BUFFERED",
	}
	WriteStream_Type_value = map[string]int32{
		"TYPE_UNSPECIFIED": 0,
		"COMMITTED":        1,
		"PENDING":          2,
		"BUFFERED":         3,
	}
)

func (x WriteStream_Type) Enum() *WriteStream_Type {
	p := new(WriteStream_Type)
	*p = x
	return p
}

func (x WriteStream_Type) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (WriteStream_Type) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[3].Descriptor()
}

func (WriteStream_Type) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[3]
}

func (x WriteStream_Type) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use WriteStream_Type.Descriptor instead.
func (WriteStream_Type) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{2, 0}
}

// Mode enum of the stream.
type WriteStream_WriteMode int32

const (
	// Unknown type.
	WriteStream_WRITE_MODE_UNSPECIFIED WriteStream_WriteMode = 0
	// Insert new records into the table.
	// It is the default value if customers do not specify it.
	WriteStream_INSERT WriteStream_WriteMode = 1
)

// Enum value maps for WriteStream_WriteMode.
var (
	WriteStream_WriteMode_name = map[int32]string{
		0: "WRITE_MODE_UNSPECIFIED",
		1: "INSERT",
	}
	WriteStream_WriteMode_value = map[string]int32{
		"WRITE_MODE_UNSPECIFIED": 0,
		"INSERT":                 1,
	}
)

func (x WriteStream_WriteMode) Enum() *WriteStream_WriteMode {
	p := new(WriteStream_WriteMode)
	*p = x
	return p
}

func (x WriteStream_WriteMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (WriteStream_WriteMode) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[4].Descriptor()
}

func (WriteStream_WriteMode) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes[4]
}

func (x WriteStream_WriteMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use WriteStream_WriteMode.Descriptor instead.
func (WriteStream_WriteMode) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{2, 1}
}

// Information about the ReadSession.
type ReadSession struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Output only. Unique identifier for the session, in the form
	// `projects/{project_id}/locations/{location}/sessions/{session_id}`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Output only. Time at which the session becomes invalid. After this time,
	// subsequent requests to read this Session will return errors. The
	// expire_time is automatically assigned and currently cannot be specified or
	// updated.
	ExpireTime *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=expire_time,json=expireTime,proto3" json:"expire_time,omitempty"`
	// Immutable. Data format of the output data. DATA_FORMAT_UNSPECIFIED not
	// supported.
	DataFormat DataFormat `protobuf:"varint,3,opt,name=data_format,json=dataFormat,proto3,enum=google.cloud.bigquery.storage.v1.DataFormat" json:"data_format,omitempty"`
	// The schema for the read. If read_options.selected_fields is set, the
	// schema may be different from the table schema as it will only contain
	// the selected fields.
	//
	// Types that are valid to be assigned to Schema:
	//
	//	*ReadSession_AvroSchema
	//	*ReadSession_ArrowSchema
	Schema isReadSession_Schema `protobuf_oneof:"schema"`
	// Immutable. Table that this ReadSession is reading from, in the form
	// `projects/{project_id}/datasets/{dataset_id}/tables/{table_id}`
	Table string `protobuf:"bytes,6,opt,name=table,proto3" json:"table,omitempty"`
	// Optional. Any modifiers which are applied when reading from the specified
	// table.
	TableModifiers *ReadSession_TableModifiers `protobuf:"bytes,7,opt,name=table_modifiers,json=tableModifiers,proto3" json:"table_modifiers,omitempty"`
	// Optional. Read options for this session (e.g. column selection, filters).
	ReadOptions *ReadSession_TableReadOptions `protobuf:"bytes,8,opt,name=read_options,json=readOptions,proto3" json:"read_options,omitempty"`
	// Output only. A list of streams created with the session.
	//
	// At least one stream is created with the session. In the future, larger
	// request_stream_count values *may* result in this list being unpopulated,
	// in that case, the user will need to use a List method to get the streams
	// instead, which is not yet available.
	Streams []*ReadStream `protobuf:"bytes,10,rep,name=streams,proto3" json:"streams,omitempty"`
	// Output only. An estimate on the number of bytes this session will scan when
	// all streams are completely consumed. This estimate is based on
	// metadata from the table which might be incomplete or stale.
	EstimatedTotalBytesScanned int64 `protobuf:"varint,12,opt,name=estimated_total_bytes_scanned,json=estimatedTotalBytesScanned,proto3" json:"estimated_total_bytes_scanned,omitempty"`
	// Output only. A pre-projected estimate of the total physical size of files
	// (in bytes) that this session will scan when all streams are consumed. This
	// estimate is independent of the selected columns and can be based on
	// incomplete or stale metadata from the table.  This field is only set for
	// BigLake tables.
	EstimatedTotalPhysicalFileSize int64 `protobuf:"varint,15,opt,name=estimated_total_physical_file_size,json=estimatedTotalPhysicalFileSize,proto3" json:"estimated_total_physical_file_size,omitempty"`
	// Output only. An estimate on the number of rows present in this session's
	// streams. This estimate is based on metadata from the table which might be
	// incomplete or stale.
	EstimatedRowCount int64 `protobuf:"varint,14,opt,name=estimated_row_count,json=estimatedRowCount,proto3" json:"estimated_row_count,omitempty"`
	// Optional. ID set by client to annotate a session identity.  This does not
	// need to be strictly unique, but instead the same ID should be used to group
	// logically connected sessions (e.g. All using the same ID for all sessions
	// needed to complete a Spark SQL query is reasonable).
	//
	// Maximum length is 256 bytes.
	TraceId       string `protobuf:"bytes,13,opt,name=trace_id,json=traceId,proto3" json:"trace_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ReadSession) Reset() {
	*x = ReadSession{}
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadSession) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadSession) ProtoMessage() {}

func (x *ReadSession) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadSession.ProtoReflect.Descriptor instead.
func (*ReadSession) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{0}
}

func (x *ReadSession) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ReadSession) GetExpireTime() *timestamppb.Timestamp {
	if x != nil {
		return x.ExpireTime
	}
	return nil
}

func (x *ReadSession) GetDataFormat() DataFormat {
	if x != nil {
		return x.DataFormat
	}
	return DataFormat_DATA_FORMAT_UNSPECIFIED
}

func (x *ReadSession) GetSchema() isReadSession_Schema {
	if x != nil {
		return x.Schema
	}
	return nil
}

func (x *ReadSession) GetAvroSchema() *AvroSchema {
	if x != nil {
		if x, ok := x.Schema.(*ReadSession_AvroSchema); ok {
			return x.AvroSchema
		}
	}
	return nil
}

func (x *ReadSession) GetArrowSchema() *ArrowSchema {
	if x != nil {
		if x, ok := x.Schema.(*ReadSession_ArrowSchema); ok {
			return x.ArrowSchema
		}
	}
	return nil
}

func (x *ReadSession) GetTable() string {
	if x != nil {
		return x.Table
	}
	return ""
}

func (x *ReadSession) GetTableModifiers() *ReadSession_TableModifiers {
	if x != nil {
		return x.TableModifiers
	}
	return nil
}

func (x *ReadSession) GetReadOptions() *ReadSession_TableReadOptions {
	if x != nil {
		return x.ReadOptions
	}
	return nil
}

func (x *ReadSession) GetStreams() []*ReadStream {
	if x != nil {
		return x.Streams
	}
	return nil
}

func (x *ReadSession) GetEstimatedTotalBytesScanned() int64 {
	if x != nil {
		return x.EstimatedTotalBytesScanned
	}
	return 0
}

func (x *ReadSession) GetEstimatedTotalPhysicalFileSize() int64 {
	if x != nil {
		return x.EstimatedTotalPhysicalFileSize
	}
	return 0
}

func (x *ReadSession) GetEstimatedRowCount() int64 {
	if x != nil {
		return x.EstimatedRowCount
	}
	return 0
}

func (x *ReadSession) GetTraceId() string {
	if x != nil {
		return x.TraceId
	}
	return ""
}

type isReadSession_Schema interface {
	isReadSession_Schema()
}

type ReadSession_AvroSchema struct {
	// Output only. Avro schema.
	AvroSchema *AvroSchema `protobuf:"bytes,4,opt,name=avro_schema,json=avroSchema,proto3,oneof"`
}

type ReadSession_ArrowSchema struct {
	// Output only. Arrow schema.
	ArrowSchema *ArrowSchema `protobuf:"bytes,5,opt,name=arrow_schema,json=arrowSchema,proto3,oneof"`
}

func (*ReadSession_AvroSchema) isReadSession_Schema() {}

func (*ReadSession_ArrowSchema) isReadSession_Schema() {}

// Information about a single stream that gets data out of the storage system.
// Most of the information about `ReadStream` instances is aggregated, making
// `ReadStream` lightweight.
type ReadStream struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Output only. Name of the stream, in the form
	// `projects/{project_id}/locations/{location}/sessions/{session_id}/streams/{stream_id}`.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ReadStream) Reset() {
	*x = ReadStream{}
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadStream) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadStream) ProtoMessage() {}

func (x *ReadStream) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadStream.ProtoReflect.Descriptor instead.
func (*ReadStream) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{1}
}

func (x *ReadStream) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

// Information about a single stream that gets data inside the storage system.
type WriteStream struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Output only. Name of the stream, in the form
	// `projects/{project}/datasets/{dataset}/tables/{table}/streams/{stream}`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Immutable. Type of the stream.
	Type WriteStream_Type `protobuf:"varint,2,opt,name=type,proto3,enum=google.cloud.bigquery.storage.v1.WriteStream_Type" json:"type,omitempty"`
	// Output only. Create time of the stream. For the _default stream, this is
	// the creation_time of the table.
	CreateTime *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=create_time,json=createTime,proto3" json:"create_time,omitempty"`
	// Output only. Commit time of the stream.
	// If a stream is of `COMMITTED` type, then it will have a commit_time same as
	// `create_time`. If the stream is of `PENDING` type, empty commit_time
	// means it is not committed.
	CommitTime *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=commit_time,json=commitTime,proto3" json:"commit_time,omitempty"`
	// Output only. The schema of the destination table. It is only returned in
	// `CreateWriteStream` response. Caller should generate data that's
	// compatible with this schema to send in initial `AppendRowsRequest`.
	// The table schema could go out of date during the life time of the stream.
	TableSchema *TableSchema `protobuf:"bytes,5,opt,name=table_schema,json=tableSchema,proto3" json:"table_schema,omitempty"`
	// Immutable. Mode of the stream.
	WriteMode WriteStream_WriteMode `protobuf:"varint,7,opt,name=write_mode,json=writeMode,proto3,enum=google.cloud.bigquery.storage.v1.WriteStream_WriteMode" json:"write_mode,omitempty"`
	// Output only. The geographic location where the stream's dataset resides.
	// See https://cloud.google.com/bigquery/docs/locations for supported
	// locations.
	Location      string `protobuf:"bytes,8,opt,name=location,proto3" json:"location,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *WriteStream) Reset() {
	*x = WriteStream{}
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WriteStream) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WriteStream) ProtoMessage() {}

func (x *WriteStream) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WriteStream.ProtoReflect.Descriptor instead.
func (*WriteStream) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{2}
}

func (x *WriteStream) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *WriteStream) GetType() WriteStream_Type {
	if x != nil {
		return x.Type
	}
	return WriteStream_TYPE_UNSPECIFIED
}

func (x *WriteStream) GetCreateTime() *timestamppb.Timestamp {
	if x != nil {
		return x.CreateTime
	}
	return nil
}

func (x *WriteStream) GetCommitTime() *timestamppb.Timestamp {
	if x != nil {
		return x.CommitTime
	}
	return nil
}

func (x *WriteStream) GetTableSchema() *TableSchema {
	if x != nil {
		return x.TableSchema
	}
	return nil
}

func (x *WriteStream) GetWriteMode() WriteStream_WriteMode {
	if x != nil {
		return x.WriteMode
	}
	return WriteStream_WRITE_MODE_UNSPECIFIED
}

func (x *WriteStream) GetLocation() string {
	if x != nil {
		return x.Location
	}
	return ""
}

// Additional attributes when reading a table.
type ReadSession_TableModifiers struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The snapshot time of the table. If not set, interpreted as now.
	SnapshotTime  *timestamppb.Timestamp `protobuf:"bytes,1,opt,name=snapshot_time,json=snapshotTime,proto3" json:"snapshot_time,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ReadSession_TableModifiers) Reset() {
	*x = ReadSession_TableModifiers{}
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadSession_TableModifiers) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadSession_TableModifiers) ProtoMessage() {}

func (x *ReadSession_TableModifiers) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadSession_TableModifiers.ProtoReflect.Descriptor instead.
func (*ReadSession_TableModifiers) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{0, 0}
}

func (x *ReadSession_TableModifiers) GetSnapshotTime() *timestamppb.Timestamp {
	if x != nil {
		return x.SnapshotTime
	}
	return nil
}

// Options dictating how we read a table.
type ReadSession_TableReadOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional. The names of the fields in the table to be returned. If no
	// field names are specified, then all fields in the table are returned.
	//
	// Nested fields -- the child elements of a STRUCT field -- can be selected
	// individually using their fully-qualified names, and will be returned as
	// record fields containing only the selected nested fields. If a STRUCT
	// field is specified in the selected fields list, all of the child elements
	// will be returned.
	//
	// As an example, consider a table with the following schema:
	//
	//	{
	//	    "name": "struct_field",
	//	    "type": "RECORD",
	//	    "mode": "NULLABLE",
	//	    "fields": [
	//	        {
	//	            "name": "string_field1",
	//	            "type": "STRING",
	//
	// .              "mode": "NULLABLE"
	//
	//	        },
	//	        {
	//	            "name": "string_field2",
	//	            "type": "STRING",
	//	            "mode": "NULLABLE"
	//	        }
	//	    ]
	//	}
	//
	// Specifying "struct_field" in the selected fields list will result in a
	// read session schema with the following logical structure:
	//
	//	struct_field {
	//	    string_field1
	//	    string_field2
	//	}
	//
	// Specifying "struct_field.string_field1" in the selected fields list will
	// result in a read session schema with the following logical structure:
	//
	//	struct_field {
	//	    string_field1
	//	}
	//
	// The order of the fields in the read session schema is derived from the
	// table schema and does not correspond to the order in which the fields are
	// specified in this list.
	SelectedFields []string `protobuf:"bytes,1,rep,name=selected_fields,json=selectedFields,proto3" json:"selected_fields,omitempty"`
	// SQL text filtering statement, similar to a WHERE clause in a query.
	// Aggregates are not supported.
	//
	// Examples: "int_field > 5"
	//
	//	"date_field = CAST('2014-9-27' as DATE)"
	//	"nullable_field is not NULL"
	//	"st_equals(geo_field, st_geofromtext("POINT(2, 2)"))"
	//	"numeric_field BETWEEN 1.0 AND 5.0"
	//
	// Restricted to a maximum length for 1 MB.
	RowRestriction string `protobuf:"bytes,2,opt,name=row_restriction,json=rowRestriction,proto3" json:"row_restriction,omitempty"`
	// Types that are valid to be assigned to OutputFormatSerializationOptions:
	//
	//	*ReadSession_TableReadOptions_ArrowSerializationOptions
	//	*ReadSession_TableReadOptions_AvroSerializationOptions
	OutputFormatSerializationOptions isReadSession_TableReadOptions_OutputFormatSerializationOptions `protobuf_oneof:"output_format_serialization_options"`
	// Optional. Specifies a table sampling percentage. Specifically, the query
	// planner will use TABLESAMPLE SYSTEM (sample_percentage PERCENT). The
	// sampling percentage is applied at the data block granularity. It will
	// randomly choose for each data block whether to read the rows in that data
	// block. For more details, see
	// https://cloud.google.com/bigquery/docs/table-sampling)
	SamplePercentage *float64 `protobuf:"fixed64,5,opt,name=sample_percentage,json=samplePercentage,proto3,oneof" json:"sample_percentage,omitempty"`
	// Optional. Set response_compression_codec when creating a read session to
	// enable application-level compression of ReadRows responses.
	ResponseCompressionCodec *ReadSession_TableReadOptions_ResponseCompressionCodec `protobuf:"varint,6,opt,name=response_compression_codec,json=responseCompressionCodec,proto3,enum=google.cloud.bigquery.storage.v1.ReadSession_TableReadOptions_ResponseCompressionCodec,oneof" json:"response_compression_codec,omitempty"`
	unknownFields            protoimpl.UnknownFields
	sizeCache                protoimpl.SizeCache
}

func (x *ReadSession_TableReadOptions) Reset() {
	*x = ReadSession_TableReadOptions{}
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadSession_TableReadOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadSession_TableReadOptions) ProtoMessage() {}

func (x *ReadSession_TableReadOptions) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadSession_TableReadOptions.ProtoReflect.Descriptor instead.
func (*ReadSession_TableReadOptions) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP(), []int{0, 1}
}

func (x *ReadSession_TableReadOptions) GetSelectedFields() []string {
	if x != nil {
		return x.SelectedFields
	}
	return nil
}

func (x *ReadSession_TableReadOptions) GetRowRestriction() string {
	if x != nil {
		return x.RowRestriction
	}
	return ""
}

func (x *ReadSession_TableReadOptions) GetOutputFormatSerializationOptions() isReadSession_TableReadOptions_OutputFormatSerializationOptions {
	if x != nil {
		return x.OutputFormatSerializationOptions
	}
	return nil
}

func (x *ReadSession_TableReadOptions) GetArrowSerializationOptions() *ArrowSerializationOptions {
	if x != nil {
		if x, ok := x.OutputFormatSerializationOptions.(*ReadSession_TableReadOptions_ArrowSerializationOptions); ok {
			return x.ArrowSerializationOptions
		}
	}
	return nil
}

func (x *ReadSession_TableReadOptions) GetAvroSerializationOptions() *AvroSerializationOptions {
	if x != nil {
		if x, ok := x.OutputFormatSerializationOptions.(*ReadSession_TableReadOptions_AvroSerializationOptions); ok {
			return x.AvroSerializationOptions
		}
	}
	return nil
}

func (x *ReadSession_TableReadOptions) GetSamplePercentage() float64 {
	if x != nil && x.SamplePercentage != nil {
		return *x.SamplePercentage
	}
	return 0
}

func (x *ReadSession_TableReadOptions) GetResponseCompressionCodec() ReadSession_TableReadOptions_ResponseCompressionCodec {
	if x != nil && x.ResponseCompressionCodec != nil {
		return *x.ResponseCompressionCodec
	}
	return ReadSession_TableReadOptions_RESPONSE_COMPRESSION_CODEC_UNSPECIFIED
}

type isReadSession_TableReadOptions_OutputFormatSerializationOptions interface {
	isReadSession_TableReadOptions_OutputFormatSerializationOptions()
}

type ReadSession_TableReadOptions_ArrowSerializationOptions struct {
	// Optional. Options specific to the Apache Arrow output format.
	ArrowSerializationOptions *ArrowSerializationOptions `protobuf:"bytes,3,opt,name=arrow_serialization_options,json=arrowSerializationOptions,proto3,oneof"`
}

type ReadSession_TableReadOptions_AvroSerializationOptions struct {
	// Optional. Options specific to the Apache Avro output format
	AvroSerializationOptions *AvroSerializationOptions `protobuf:"bytes,4,opt,name=avro_serialization_options,json=avroSerializationOptions,proto3,oneof"`
}

func (*ReadSession_TableReadOptions_ArrowSerializationOptions) isReadSession_TableReadOptions_OutputFormatSerializationOptions() {
}

func (*ReadSession_TableReadOptions_AvroSerializationOptions) isReadSession_TableReadOptions_OutputFormatSerializationOptions() {
}

var File_google_cloud_bigquery_storage_v1_stream_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_stream_proto_rawDesc = "" +
	"\n" +
	"-google/cloud/bigquery/storage/v1/stream.proto\x12 google.cloud.bigquery.storage.v1\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a,google/cloud/bigquery/storage/v1/arrow.proto\x1a+google/cloud/bigquery/storage/v1/avro.proto\x1a,google/cloud/bigquery/storage/v1/table.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x8f\x0f\n" +
	"\vReadSession\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\x03R\x04name\x12@\n" +
	"\vexpire_time\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampB\x03\xe0A\x03R\n" +
	"expireTime\x12R\n" +
	"\vdata_format\x18\x03 \x01(\x0e2,.google.cloud.bigquery.storage.v1.DataFormatB\x03\xe0A\x05R\n" +
	"dataFormat\x12T\n" +
	"\vavro_schema\x18\x04 \x01(\v2,.google.cloud.bigquery.storage.v1.AvroSchemaB\x03\xe0A\x03H\x00R\n" +
	"avroSchema\x12W\n" +
	"\farrow_schema\x18\x05 \x01(\v2-.google.cloud.bigquery.storage.v1.ArrowSchemaB\x03\xe0A\x03H\x00R\varrowSchema\x12;\n" +
	"\x05table\x18\x06 \x01(\tB%\xe0A\x05\xfaA\x1f\n" +
	"\x1dbigquery.googleapis.com/TableR\x05table\x12j\n" +
	"\x0ftable_modifiers\x18\a \x01(\v2<.google.cloud.bigquery.storage.v1.ReadSession.TableModifiersB\x03\xe0A\x01R\x0etableModifiers\x12f\n" +
	"\fread_options\x18\b \x01(\v2>.google.cloud.bigquery.storage.v1.ReadSession.TableReadOptionsB\x03\xe0A\x01R\vreadOptions\x12K\n" +
	"\astreams\x18\n" +
	" \x03(\v2,.google.cloud.bigquery.storage.v1.ReadStreamB\x03\xe0A\x03R\astreams\x12F\n" +
	"\x1destimated_total_bytes_scanned\x18\f \x01(\x03B\x03\xe0A\x03R\x1aestimatedTotalBytesScanned\x12O\n" +
	"\"estimated_total_physical_file_size\x18\x0f \x01(\x03B\x03\xe0A\x03R\x1eestimatedTotalPhysicalFileSize\x123\n" +
	"\x13estimated_row_count\x18\x0e \x01(\x03B\x03\xe0A\x03R\x11estimatedRowCount\x12\x1e\n" +
	"\btrace_id\x18\r \x01(\tB\x03\xe0A\x01R\atraceId\x1aQ\n" +
	"\x0eTableModifiers\x12?\n" +
	"\rsnapshot_time\x18\x01 \x01(\v2\x1a.google.protobuf.TimestampR\fsnapshotTime\x1a\x8b\x06\n" +
	"\x10TableReadOptions\x12'\n" +
	"\x0fselected_fields\x18\x01 \x03(\tR\x0eselectedFields\x12'\n" +
	"\x0frow_restriction\x18\x02 \x01(\tR\x0erowRestriction\x12\x82\x01\n" +
	"\x1barrow_serialization_options\x18\x03 \x01(\v2;.google.cloud.bigquery.storage.v1.ArrowSerializationOptionsB\x03\xe0A\x01H\x00R\x19arrowSerializationOptions\x12\x7f\n" +
	"\x1aavro_serialization_options\x18\x04 \x01(\v2:.google.cloud.bigquery.storage.v1.AvroSerializationOptionsB\x03\xe0A\x01H\x00R\x18avroSerializationOptions\x125\n" +
	"\x11sample_percentage\x18\x05 \x01(\x01B\x03\xe0A\x01H\x01R\x10samplePercentage\x88\x01\x01\x12\x9f\x01\n" +
	"\x1aresponse_compression_codec\x18\x06 \x01(\x0e2W.google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.ResponseCompressionCodecB\x03\xe0A\x01H\x02R\x18responseCompressionCodec\x88\x01\x01\"j\n" +
	"\x18ResponseCompressionCodec\x12*\n" +
	"&RESPONSE_COMPRESSION_CODEC_UNSPECIFIED\x10\x00\x12\"\n" +
	"\x1eRESPONSE_COMPRESSION_CODEC_LZ4\x10\x02B%\n" +
	"#output_format_serialization_optionsB\x14\n" +
	"\x12_sample_percentageB\x1d\n" +
	"\x1b_response_compression_codec:k\xeaAh\n" +
	"*bigquerystorage.googleapis.com/ReadSession\x12:projects/{project}/locations/{location}/sessions/{session}B\b\n" +
	"\x06schema\"\xa2\x01\n" +
	"\n" +
	"ReadStream\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\x03R\x04name:{\xeaAx\n" +
	")bigquerystorage.googleapis.com/ReadStream\x12Kprojects/{project}/locations/{location}/sessions/{session}/streams/{stream}\"\xc1\x05\n" +
	"\vWriteStream\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\x03R\x04name\x12K\n" +
	"\x04type\x18\x02 \x01(\x0e22.google.cloud.bigquery.storage.v1.WriteStream.TypeB\x03\xe0A\x05R\x04type\x12@\n" +
	"\vcreate_time\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampB\x03\xe0A\x03R\n" +
	"createTime\x12@\n" +
	"\vcommit_time\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampB\x03\xe0A\x03R\n" +
	"commitTime\x12U\n" +
	"\ftable_schema\x18\x05 \x01(\v2-.google.cloud.bigquery.storage.v1.TableSchemaB\x03\xe0A\x03R\vtableSchema\x12[\n" +
	"\n" +
	"write_mode\x18\a \x01(\x0e27.google.cloud.bigquery.storage.v1.WriteStream.WriteModeB\x03\xe0A\x05R\twriteMode\x12\x1f\n" +
	"\blocation\x18\b \x01(\tB\x03\xe0A\x03R\blocation\"F\n" +
	"\x04Type\x12\x14\n" +
	"\x10TYPE_UNSPECIFIED\x10\x00\x12\r\n" +
	"\tCOMMITTED\x10\x01\x12\v\n" +
	"\aPENDING\x10\x02\x12\f\n" +
	"\bBUFFERED\x10\x03\"3\n" +
	"\tWriteMode\x12\x1a\n" +
	"\x16WRITE_MODE_UNSPECIFIED\x10\x00\x12\n" +
	"\n" +
	"\x06INSERT\x10\x01:v\xeaAs\n" +
	"*bigquerystorage.googleapis.com/WriteStream\x12Eprojects/{project}/datasets/{dataset}/tables/{table}/streams/{stream}*>\n" +
	"\n" +
	"DataFormat\x12\x1b\n" +
	"\x17DATA_FORMAT_UNSPECIFIED\x10\x00\x12\b\n" +
	"\x04AVRO\x10\x01\x12\t\n" +
	"\x05ARROW\x10\x02*I\n" +
	"\x0fWriteStreamView\x12!\n" +
	"\x1dWRITE_STREAM_VIEW_UNSPECIFIED\x10\x00\x12\t\n" +
	"\x05BASIC\x10\x01\x12\b\n" +
	"\x04FULL\x10\x02B\xbb\x01\n" +
	"$com.google.cloud.bigquery.storage.v1B\vStreamProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var (
	file_google_cloud_bigquery_storage_v1_stream_proto_rawDescOnce sync.Once
	file_google_cloud_bigquery_storage_v1_stream_proto_rawDescData []byte
)

func file_google_cloud_bigquery_storage_v1_stream_proto_rawDescGZIP() []byte {
	file_google_cloud_bigquery_storage_v1_stream_proto_rawDescOnce.Do(func() {
		file_google_cloud_bigquery_storage_v1_stream_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_stream_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_stream_proto_rawDesc)))
	})
	return file_google_cloud_bigquery_storage_v1_stream_proto_rawDescData
}

var file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes = make([]protoimpl.EnumInfo, 5)
var file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_google_cloud_bigquery_storage_v1_stream_proto_goTypes = []any{
	(DataFormat)(0),      // 0: google.cloud.bigquery.storage.v1.DataFormat
	(WriteStreamView)(0), // 1: google.cloud.bigquery.storage.v1.WriteStreamView
	(ReadSession_TableReadOptions_ResponseCompressionCodec)(0), // 2: google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.ResponseCompressionCodec
	(WriteStream_Type)(0),                // 3: google.cloud.bigquery.storage.v1.WriteStream.Type
	(WriteStream_WriteMode)(0),           // 4: google.cloud.bigquery.storage.v1.WriteStream.WriteMode
	(*ReadSession)(nil),                  // 5: google.cloud.bigquery.storage.v1.ReadSession
	(*ReadStream)(nil),                   // 6: google.cloud.bigquery.storage.v1.ReadStream
	(*WriteStream)(nil),                  // 7: google.cloud.bigquery.storage.v1.WriteStream
	(*ReadSession_TableModifiers)(nil),   // 8: google.cloud.bigquery.storage.v1.ReadSession.TableModifiers
	(*ReadSession_TableReadOptions)(nil), // 9: google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions
	(*timestamppb.Timestamp)(nil),        // 10: google.protobuf.Timestamp
	(*AvroSchema)(nil),                   // 11: google.cloud.bigquery.storage.v1.AvroSchema
	(*ArrowSchema)(nil),                  // 12: google.cloud.bigquery.storage.v1.ArrowSchema
	(*TableSchema)(nil),                  // 13: google.cloud.bigquery.storage.v1.TableSchema
	(*ArrowSerializationOptions)(nil),    // 14: google.cloud.bigquery.storage.v1.ArrowSerializationOptions
	(*AvroSerializationOptions)(nil),     // 15: google.cloud.bigquery.storage.v1.AvroSerializationOptions
}
var file_google_cloud_bigquery_storage_v1_stream_proto_depIdxs = []int32{
	10, // 0: google.cloud.bigquery.storage.v1.ReadSession.expire_time:type_name -> google.protobuf.Timestamp
	0,  // 1: google.cloud.bigquery.storage.v1.ReadSession.data_format:type_name -> google.cloud.bigquery.storage.v1.DataFormat
	11, // 2: google.cloud.bigquery.storage.v1.ReadSession.avro_schema:type_name -> google.cloud.bigquery.storage.v1.AvroSchema
	12, // 3: google.cloud.bigquery.storage.v1.ReadSession.arrow_schema:type_name -> google.cloud.bigquery.storage.v1.ArrowSchema
	8,  // 4: google.cloud.bigquery.storage.v1.ReadSession.table_modifiers:type_name -> google.cloud.bigquery.storage.v1.ReadSession.TableModifiers
	9,  // 5: google.cloud.bigquery.storage.v1.ReadSession.read_options:type_name -> google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions
	6,  // 6: google.cloud.bigquery.storage.v1.ReadSession.streams:type_name -> google.cloud.bigquery.storage.v1.ReadStream
	3,  // 7: google.cloud.bigquery.storage.v1.WriteStream.type:type_name -> google.cloud.bigquery.storage.v1.WriteStream.Type
	10, // 8: google.cloud.bigquery.storage.v1.WriteStream.create_time:type_name -> google.protobuf.Timestamp
	10, // 9: google.cloud.bigquery.storage.v1.WriteStream.commit_time:type_name -> google.protobuf.Timestamp
	13, // 10: google.cloud.bigquery.storage.v1.WriteStream.table_schema:type_name -> google.cloud.bigquery.storage.v1.TableSchema
	4,  // 11: google.cloud.bigquery.storage.v1.WriteStream.write_mode:type_name -> google.cloud.bigquery.storage.v1.WriteStream.WriteMode
	10, // 12: google.cloud.bigquery.storage.v1.ReadSession.TableModifiers.snapshot_time:type_name -> google.protobuf.Timestamp
	14, // 13: google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.arrow_serialization_options:type_name -> google.cloud.bigquery.storage.v1.ArrowSerializationOptions
	15, // 14: google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.avro_serialization_options:type_name -> google.cloud.bigquery.storage.v1.AvroSerializationOptions
	2,  // 15: google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.response_compression_codec:type_name -> google.cloud.bigquery.storage.v1.ReadSession.TableReadOptions.ResponseCompressionCodec
	16, // [16:16] is the sub-list for method output_type
	16, // [16:16] is the sub-list for method input_type
	16, // [16:16] is the sub-list for extension type_name
	16, // [16:16] is the sub-list for extension extendee
	0,  // [0:16] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_stream_proto_init() }
func file_google_cloud_bigquery_storage_v1_stream_proto_init() {
	if File_google_cloud_bigquery_storage_v1_stream_proto != nil {
		return
	}
	file_google_cloud_bigquery_storage_v1_arrow_proto_init()
	file_google_cloud_bigquery_storage_v1_avro_proto_init()
	file_google_cloud_bigquery_storage_v1_table_proto_init()
	file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[0].OneofWrappers = []any{
		(*ReadSession_AvroSchema)(nil),
		(*ReadSession_ArrowSchema)(nil),
	}
	file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes[4].OneofWrappers = []any{
		(*ReadSession_TableReadOptions_ArrowSerializationOptions)(nil),
		(*ReadSession_TableReadOptions_AvroSerializationOptions)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_stream_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_stream_proto_rawDesc)),
			NumEnums:      5,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_stream_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_stream_proto_depIdxs,
		EnumInfos:         file_google_cloud_bigquery_storage_v1_stream_proto_enumTypes,
		MessageInfos:      file_google_cloud_bigquery_storage_v1_stream_proto_msgTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_stream_proto = out.File
	file_google_cloud_bigquery_storage_v1_stream_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_stream_proto_depIdxs = nil
}
