// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/protobuf.proto

package storagepb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// ProtoSchema describes the schema of the serialized protocol buffer data rows.
type ProtoSchema struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Descriptor for input message.  The provided descriptor must be self
	// contained, such that data rows sent can be fully decoded using only the
	// single descriptor.  For data rows that are compositions of multiple
	// independent messages, this means the descriptor may need to be transformed
	// to only use nested types:
	// https://developers.google.com/protocol-buffers/docs/proto#nested
	//
	// For additional information for how proto types and values map onto BigQuery
	// see: https://cloud.google.com/bigquery/docs/write-api#data_type_conversions
	ProtoDescriptor *descriptorpb.DescriptorProto `protobuf:"bytes,1,opt,name=proto_descriptor,json=protoDescriptor,proto3" json:"proto_descriptor,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *ProtoSchema) Reset() {
	*x = ProtoSchema{}
	mi := &file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProtoSchema) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProtoSchema) ProtoMessage() {}

func (x *ProtoSchema) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProtoSchema.ProtoReflect.Descriptor instead.
func (*ProtoSchema) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescGZIP(), []int{0}
}

func (x *ProtoSchema) GetProtoDescriptor() *descriptorpb.DescriptorProto {
	if x != nil {
		return x.ProtoDescriptor
	}
	return nil
}

type ProtoRows struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A sequence of rows serialized as a Protocol Buffer.
	//
	// See https://developers.google.com/protocol-buffers/docs/overview for more
	// information on deserializing this field.
	SerializedRows [][]byte `protobuf:"bytes,1,rep,name=serialized_rows,json=serializedRows,proto3" json:"serialized_rows,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ProtoRows) Reset() {
	*x = ProtoRows{}
	mi := &file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProtoRows) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProtoRows) ProtoMessage() {}

func (x *ProtoRows) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProtoRows.ProtoReflect.Descriptor instead.
func (*ProtoRows) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescGZIP(), []int{1}
}

func (x *ProtoRows) GetSerializedRows() [][]byte {
	if x != nil {
		return x.SerializedRows
	}
	return nil
}

var File_google_cloud_bigquery_storage_v1_protobuf_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDesc = "" +
	"\n" +
	"/google/cloud/bigquery/storage/v1/protobuf.proto\x12 google.cloud.bigquery.storage.v1\x1a google/protobuf/descriptor.proto\"Z\n" +
	"\vProtoSchema\x12K\n" +
	"\x10proto_descriptor\x18\x01 \x01(\v2 .google.protobuf.DescriptorProtoR\x0fprotoDescriptor\"4\n" +
	"\tProtoRows\x12'\n" +
	"\x0fserialized_rows\x18\x01 \x03(\fR\x0eserializedRowsB\xbd\x01\n" +
	"$com.google.cloud.bigquery.storage.v1B\rProtoBufProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var (
	file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescOnce sync.Once
	file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescData []byte
)

func file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescGZIP() []byte {
	file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescOnce.Do(func() {
		file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDesc)))
	})
	return file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDescData
}

var file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_google_cloud_bigquery_storage_v1_protobuf_proto_goTypes = []any{
	(*ProtoSchema)(nil),                  // 0: google.cloud.bigquery.storage.v1.ProtoSchema
	(*ProtoRows)(nil),                    // 1: google.cloud.bigquery.storage.v1.ProtoRows
	(*descriptorpb.DescriptorProto)(nil), // 2: google.protobuf.DescriptorProto
}
var file_google_cloud_bigquery_storage_v1_protobuf_proto_depIdxs = []int32{
	2, // 0: google.cloud.bigquery.storage.v1.ProtoSchema.proto_descriptor:type_name -> google.protobuf.DescriptorProto
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_protobuf_proto_init() }
func file_google_cloud_bigquery_storage_v1_protobuf_proto_init() {
	if File_google_cloud_bigquery_storage_v1_protobuf_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_protobuf_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_protobuf_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_protobuf_proto_depIdxs,
		MessageInfos:      file_google_cloud_bigquery_storage_v1_protobuf_proto_msgTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_protobuf_proto = out.File
	file_google_cloud_bigquery_storage_v1_protobuf_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_protobuf_proto_depIdxs = nil
}
