// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/table.proto

package storagepb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type TableFieldSchema_Type int32

const (
	// Illegal value
	TableFieldSchema_TYPE_UNSPECIFIED TableFieldSchema_Type = 0
	// 64K, UTF8
	TableFieldSchema_STRING TableFieldSchema_Type = 1
	// 64-bit signed
	TableFieldSchema_INT64 TableFieldSchema_Type = 2
	// 64-bit IEEE floating point
	TableFieldSchema_DOUBLE TableFieldSchema_Type = 3
	// Aggregate type
	TableFieldSchema_STRUCT TableFieldSchema_Type = 4
	// 64K, Binary
	TableFieldSchema_BYTES TableFieldSchema_Type = 5
	// 2-valued
	TableFieldSchema_BOOL TableFieldSchema_Type = 6
	// 64-bit signed usec since UTC epoch
	TableFieldSchema_TIMESTAMP TableFieldSchema_Type = 7
	// Civil date - Year, Month, Day
	TableFieldSchema_DATE TableFieldSchema_Type = 8
	// Civil time - Hour, Minute, Second, Microseconds
	TableFieldSchema_TIME TableFieldSchema_Type = 9
	// Combination of civil date and civil time
	TableFieldSchema_DATETIME TableFieldSchema_Type = 10
	// Geography object
	TableFieldSchema_GEOGRAPHY TableFieldSchema_Type = 11
	// Numeric value
	TableFieldSchema_NUMERIC TableFieldSchema_Type = 12
	// BigNumeric value
	TableFieldSchema_BIGNUMERIC TableFieldSchema_Type = 13
	// Interval
	TableFieldSchema_INTERVAL TableFieldSchema_Type = 14
	// JSON, String
	TableFieldSchema_JSON TableFieldSchema_Type = 15
	// RANGE
	TableFieldSchema_RANGE TableFieldSchema_Type = 16
)

// Enum value maps for TableFieldSchema_Type.
var (
	TableFieldSchema_Type_name = map[int32]string{
		0:  "TYPE_UNSPECIFIED",
		1:  "STRING",
		2:  "INT64",
		3:  "DOUBLE",
		4:  "STRUCT",
		5:  "BYTES",
		6:  "BOOL",
		7:  "TIMESTAMP",
		8:  "DATE",
		9:  "TIME",
		10: "DATETIME",
		11: "GEOGRAPHY",
		12: "NUMERIC",
		13: "BIGNUMERIC",
		14: "INTERVAL",
		15: "JSON",
		16: "RANGE",
	}
	TableFieldSchema_Type_value = map[string]int32{
		"TYPE_UNSPECIFIED": 0,
		"STRING":           1,
		"INT64":            2,
		"DOUBLE":           3,
		"STRUCT":           4,
		"BYTES":            5,
		"BOOL":             6,
		"TIMESTAMP":        7,
		"DATE":             8,
		"TIME":             9,
		"DATETIME":         10,
		"GEOGRAPHY":        11,
		"NUMERIC":          12,
		"BIGNUMERIC":       13,
		"INTERVAL":         14,
		"JSON":             15,
		"RANGE":            16,
	}
)

func (x TableFieldSchema_Type) Enum() *TableFieldSchema_Type {
	p := new(TableFieldSchema_Type)
	*p = x
	return p
}

func (x TableFieldSchema_Type) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (TableFieldSchema_Type) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_table_proto_enumTypes[0].Descriptor()
}

func (TableFieldSchema_Type) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_table_proto_enumTypes[0]
}

func (x TableFieldSchema_Type) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use TableFieldSchema_Type.Descriptor instead.
func (TableFieldSchema_Type) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP(), []int{1, 0}
}

type TableFieldSchema_Mode int32

const (
	// Illegal value
	TableFieldSchema_MODE_UNSPECIFIED TableFieldSchema_Mode = 0
	TableFieldSchema_NULLABLE         TableFieldSchema_Mode = 1
	TableFieldSchema_REQUIRED         TableFieldSchema_Mode = 2
	TableFieldSchema_REPEATED         TableFieldSchema_Mode = 3
)

// Enum value maps for TableFieldSchema_Mode.
var (
	TableFieldSchema_Mode_name = map[int32]string{
		0: "MODE_UNSPECIFIED",
		1: "NULLABLE",
		2: "REQUIRED",
		3: "REPEATED",
	}
	TableFieldSchema_Mode_value = map[string]int32{
		"MODE_UNSPECIFIED": 0,
		"NULLABLE":         1,
		"REQUIRED":         2,
		"REPEATED":         3,
	}
)

func (x TableFieldSchema_Mode) Enum() *TableFieldSchema_Mode {
	p := new(TableFieldSchema_Mode)
	*p = x
	return p
}

func (x TableFieldSchema_Mode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (TableFieldSchema_Mode) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_table_proto_enumTypes[1].Descriptor()
}

func (TableFieldSchema_Mode) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_table_proto_enumTypes[1]
}

func (x TableFieldSchema_Mode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use TableFieldSchema_Mode.Descriptor instead.
func (TableFieldSchema_Mode) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP(), []int{1, 1}
}

// Schema of a table. This schema is a subset of
// google.cloud.bigquery.v2.TableSchema containing information necessary to
// generate valid message to write to BigQuery.
type TableSchema struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Describes the fields in a table.
	Fields        []*TableFieldSchema `protobuf:"bytes,1,rep,name=fields,proto3" json:"fields,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TableSchema) Reset() {
	*x = TableSchema{}
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TableSchema) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TableSchema) ProtoMessage() {}

func (x *TableSchema) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TableSchema.ProtoReflect.Descriptor instead.
func (*TableSchema) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP(), []int{0}
}

func (x *TableSchema) GetFields() []*TableFieldSchema {
	if x != nil {
		return x.Fields
	}
	return nil
}

// TableFieldSchema defines a single field/column within a table schema.
type TableFieldSchema struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The field name. The name must contain only letters (a-z, A-Z),
	// numbers (0-9), or underscores (_), and must start with a letter or
	// underscore. The maximum length is 128 characters.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Required. The field data type.
	Type TableFieldSchema_Type `protobuf:"varint,2,opt,name=type,proto3,enum=google.cloud.bigquery.storage.v1.TableFieldSchema_Type" json:"type,omitempty"`
	// Optional. The field mode. The default value is NULLABLE.
	Mode TableFieldSchema_Mode `protobuf:"varint,3,opt,name=mode,proto3,enum=google.cloud.bigquery.storage.v1.TableFieldSchema_Mode" json:"mode,omitempty"`
	// Optional. Describes the nested schema fields if the type property is set to
	// STRUCT.
	Fields []*TableFieldSchema `protobuf:"bytes,4,rep,name=fields,proto3" json:"fields,omitempty"`
	// Optional. The field description. The maximum length is 1,024 characters.
	Description string `protobuf:"bytes,6,opt,name=description,proto3" json:"description,omitempty"`
	// Optional. Maximum length of values of this field for STRINGS or BYTES.
	//
	// If max_length is not specified, no maximum length constraint is imposed
	// on this field.
	//
	// If type = "STRING", then max_length represents the maximum UTF-8
	// length of strings in this field.
	//
	// If type = "BYTES", then max_length represents the maximum number of
	// bytes in this field.
	//
	// It is invalid to set this field if type is not "STRING" or "BYTES".
	MaxLength int64 `protobuf:"varint,7,opt,name=max_length,json=maxLength,proto3" json:"max_length,omitempty"`
	// Optional. Precision (maximum number of total digits in base 10) and scale
	// (maximum number of digits in the fractional part in base 10) constraints
	// for values of this field for NUMERIC or BIGNUMERIC.
	//
	// It is invalid to set precision or scale if type is not "NUMERIC" or
	// "BIGNUMERIC".
	//
	// If precision and scale are not specified, no value range constraint is
	// imposed on this field insofar as values are permitted by the type.
	//
	// Values of this NUMERIC or BIGNUMERIC field must be in this range when:
	//
	//   - Precision (P) and scale (S) are specified:
	//     [-10^(P-S) + 10^(-S), 10^(P-S) - 10^(-S)]
	//   - Precision (P) is specified but not scale (and thus scale is
	//     interpreted to be equal to zero):
	//     [-10^P + 1, 10^P - 1].
	//
	// Acceptable values for precision and scale if both are specified:
	//
	//   - If type = "NUMERIC":
	//     1 <= precision - scale <= 29 and 0 <= scale <= 9.
	//   - If type = "BIGNUMERIC":
	//     1 <= precision - scale <= 38 and 0 <= scale <= 38.
	//
	// Acceptable values for precision if only precision is specified but not
	// scale (and thus scale is interpreted to be equal to zero):
	//
	// * If type = "NUMERIC": 1 <= precision <= 29.
	// * If type = "BIGNUMERIC": 1 <= precision <= 38.
	//
	// If scale is specified but not precision, then it is invalid.
	Precision int64 `protobuf:"varint,8,opt,name=precision,proto3" json:"precision,omitempty"`
	// Optional. See documentation for precision.
	Scale int64 `protobuf:"varint,9,opt,name=scale,proto3" json:"scale,omitempty"`
	// Optional. A SQL expression to specify the [default value]
	// (https://cloud.google.com/bigquery/docs/default-values) for this field.
	DefaultValueExpression string `protobuf:"bytes,10,opt,name=default_value_expression,json=defaultValueExpression,proto3" json:"default_value_expression,omitempty"`
	// Optional. Precision (maximum number of total digits in base 10) for seconds
	// of TIMESTAMP type.
	//
	// Possible values include:
	//
	// * 6 (Default, for TIMESTAMP type with microsecond precision)
	// * 12 (For TIMESTAMP type with picosecond precision)
	TimestampPrecision *wrapperspb.Int64Value `protobuf:"bytes,27,opt,name=timestamp_precision,json=timestampPrecision,proto3" json:"timestamp_precision,omitempty"`
	// Optional. The subtype of the RANGE, if the type of this field is RANGE. If
	// the type is RANGE, this field is required. Possible values for the field
	// element type of a RANGE include:
	// * DATE
	// * DATETIME
	// * TIMESTAMP
	RangeElementType *TableFieldSchema_FieldElementType `protobuf:"bytes,11,opt,name=range_element_type,json=rangeElementType,proto3" json:"range_element_type,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *TableFieldSchema) Reset() {
	*x = TableFieldSchema{}
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TableFieldSchema) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TableFieldSchema) ProtoMessage() {}

func (x *TableFieldSchema) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TableFieldSchema.ProtoReflect.Descriptor instead.
func (*TableFieldSchema) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP(), []int{1}
}

func (x *TableFieldSchema) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *TableFieldSchema) GetType() TableFieldSchema_Type {
	if x != nil {
		return x.Type
	}
	return TableFieldSchema_TYPE_UNSPECIFIED
}

func (x *TableFieldSchema) GetMode() TableFieldSchema_Mode {
	if x != nil {
		return x.Mode
	}
	return TableFieldSchema_MODE_UNSPECIFIED
}

func (x *TableFieldSchema) GetFields() []*TableFieldSchema {
	if x != nil {
		return x.Fields
	}
	return nil
}

func (x *TableFieldSchema) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *TableFieldSchema) GetMaxLength() int64 {
	if x != nil {
		return x.MaxLength
	}
	return 0
}

func (x *TableFieldSchema) GetPrecision() int64 {
	if x != nil {
		return x.Precision
	}
	return 0
}

func (x *TableFieldSchema) GetScale() int64 {
	if x != nil {
		return x.Scale
	}
	return 0
}

func (x *TableFieldSchema) GetDefaultValueExpression() string {
	if x != nil {
		return x.DefaultValueExpression
	}
	return ""
}

func (x *TableFieldSchema) GetTimestampPrecision() *wrapperspb.Int64Value {
	if x != nil {
		return x.TimestampPrecision
	}
	return nil
}

func (x *TableFieldSchema) GetRangeElementType() *TableFieldSchema_FieldElementType {
	if x != nil {
		return x.RangeElementType
	}
	return nil
}

// Represents the type of a field element.
type TableFieldSchema_FieldElementType struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Required. The type of a field element.
	Type          TableFieldSchema_Type `protobuf:"varint,1,opt,name=type,proto3,enum=google.cloud.bigquery.storage.v1.TableFieldSchema_Type" json:"type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TableFieldSchema_FieldElementType) Reset() {
	*x = TableFieldSchema_FieldElementType{}
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TableFieldSchema_FieldElementType) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TableFieldSchema_FieldElementType) ProtoMessage() {}

func (x *TableFieldSchema_FieldElementType) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_table_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TableFieldSchema_FieldElementType.ProtoReflect.Descriptor instead.
func (*TableFieldSchema_FieldElementType) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP(), []int{1, 0}
}

func (x *TableFieldSchema_FieldElementType) GetType() TableFieldSchema_Type {
	if x != nil {
		return x.Type
	}
	return TableFieldSchema_TYPE_UNSPECIFIED
}

var File_google_cloud_bigquery_storage_v1_table_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_table_proto_rawDesc = "" +
	"\n" +
	",google/cloud/bigquery/storage/v1/table.proto\x12 google.cloud.bigquery.storage.v1\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1egoogle/protobuf/wrappers.proto\"Y\n" +
	"\vTableSchema\x12J\n" +
	"\x06fields\x18\x01 \x03(\v22.google.cloud.bigquery.storage.v1.TableFieldSchemaR\x06fields\"\xc4\b\n" +
	"\x10TableFieldSchema\x12\x17\n" +
	"\x04name\x18\x01 \x01(\tB\x03\xe0A\x02R\x04name\x12P\n" +
	"\x04type\x18\x02 \x01(\x0e27.google.cloud.bigquery.storage.v1.TableFieldSchema.TypeB\x03\xe0A\x02R\x04type\x12P\n" +
	"\x04mode\x18\x03 \x01(\x0e27.google.cloud.bigquery.storage.v1.TableFieldSchema.ModeB\x03\xe0A\x01R\x04mode\x12O\n" +
	"\x06fields\x18\x04 \x03(\v22.google.cloud.bigquery.storage.v1.TableFieldSchemaB\x03\xe0A\x01R\x06fields\x12%\n" +
	"\vdescription\x18\x06 \x01(\tB\x03\xe0A\x01R\vdescription\x12\"\n" +
	"\n" +
	"max_length\x18\a \x01(\x03B\x03\xe0A\x01R\tmaxLength\x12!\n" +
	"\tprecision\x18\b \x01(\x03B\x03\xe0A\x01R\tprecision\x12\x19\n" +
	"\x05scale\x18\t \x01(\x03B\x03\xe0A\x01R\x05scale\x12=\n" +
	"\x18default_value_expression\x18\n" +
	" \x01(\tB\x03\xe0A\x01R\x16defaultValueExpression\x12Q\n" +
	"\x13timestamp_precision\x18\x1b \x01(\v2\x1b.google.protobuf.Int64ValueB\x03\xe0A\x01R\x12timestampPrecision\x12v\n" +
	"\x12range_element_type\x18\v \x01(\v2C.google.cloud.bigquery.storage.v1.TableFieldSchema.FieldElementTypeB\x03\xe0A\x01R\x10rangeElementType\x1ad\n" +
	"\x10FieldElementType\x12P\n" +
	"\x04type\x18\x01 \x01(\x0e27.google.cloud.bigquery.storage.v1.TableFieldSchema.TypeB\x03\xe0A\x02R\x04type\"\xe0\x01\n" +
	"\x04Type\x12\x14\n" +
	"\x10TYPE_UNSPECIFIED\x10\x00\x12\n" +
	"\n" +
	"\x06STRING\x10\x01\x12\t\n" +
	"\x05INT64\x10\x02\x12\n" +
	"\n" +
	"\x06DOUBLE\x10\x03\x12\n" +
	"\n" +
	"\x06STRUCT\x10\x04\x12\t\n" +
	"\x05BYTES\x10\x05\x12\b\n" +
	"\x04BOOL\x10\x06\x12\r\n" +
	"\tTIMESTAMP\x10\a\x12\b\n" +
	"\x04DATE\x10\b\x12\b\n" +
	"\x04TIME\x10\t\x12\f\n" +
	"\bDATETIME\x10\n" +
	"\x12\r\n" +
	"\tGEOGRAPHY\x10\v\x12\v\n" +
	"\aNUMERIC\x10\f\x12\x0e\n" +
	"\n" +
	"BIGNUMERIC\x10\r\x12\f\n" +
	"\bINTERVAL\x10\x0e\x12\b\n" +
	"\x04JSON\x10\x0f\x12\t\n" +
	"\x05RANGE\x10\x10\"F\n" +
	"\x04Mode\x12\x14\n" +
	"\x10MODE_UNSPECIFIED\x10\x00\x12\f\n" +
	"\bNULLABLE\x10\x01\x12\f\n" +
	"\bREQUIRED\x10\x02\x12\f\n" +
	"\bREPEATED\x10\x03B\xba\x01\n" +
	"$com.google.cloud.bigquery.storage.v1B\n" +
	"TableProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var (
	file_google_cloud_bigquery_storage_v1_table_proto_rawDescOnce sync.Once
	file_google_cloud_bigquery_storage_v1_table_proto_rawDescData []byte
)

func file_google_cloud_bigquery_storage_v1_table_proto_rawDescGZIP() []byte {
	file_google_cloud_bigquery_storage_v1_table_proto_rawDescOnce.Do(func() {
		file_google_cloud_bigquery_storage_v1_table_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_table_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_table_proto_rawDesc)))
	})
	return file_google_cloud_bigquery_storage_v1_table_proto_rawDescData
}

var file_google_cloud_bigquery_storage_v1_table_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_google_cloud_bigquery_storage_v1_table_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_google_cloud_bigquery_storage_v1_table_proto_goTypes = []any{
	(TableFieldSchema_Type)(0),                // 0: google.cloud.bigquery.storage.v1.TableFieldSchema.Type
	(TableFieldSchema_Mode)(0),                // 1: google.cloud.bigquery.storage.v1.TableFieldSchema.Mode
	(*TableSchema)(nil),                       // 2: google.cloud.bigquery.storage.v1.TableSchema
	(*TableFieldSchema)(nil),                  // 3: google.cloud.bigquery.storage.v1.TableFieldSchema
	(*TableFieldSchema_FieldElementType)(nil), // 4: google.cloud.bigquery.storage.v1.TableFieldSchema.FieldElementType
	(*wrapperspb.Int64Value)(nil),             // 5: google.protobuf.Int64Value
}
var file_google_cloud_bigquery_storage_v1_table_proto_depIdxs = []int32{
	3, // 0: google.cloud.bigquery.storage.v1.TableSchema.fields:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema
	0, // 1: google.cloud.bigquery.storage.v1.TableFieldSchema.type:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema.Type
	1, // 2: google.cloud.bigquery.storage.v1.TableFieldSchema.mode:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema.Mode
	3, // 3: google.cloud.bigquery.storage.v1.TableFieldSchema.fields:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema
	5, // 4: google.cloud.bigquery.storage.v1.TableFieldSchema.timestamp_precision:type_name -> google.protobuf.Int64Value
	4, // 5: google.cloud.bigquery.storage.v1.TableFieldSchema.range_element_type:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema.FieldElementType
	0, // 6: google.cloud.bigquery.storage.v1.TableFieldSchema.FieldElementType.type:type_name -> google.cloud.bigquery.storage.v1.TableFieldSchema.Type
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_table_proto_init() }
func file_google_cloud_bigquery_storage_v1_table_proto_init() {
	if File_google_cloud_bigquery_storage_v1_table_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_table_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_table_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_table_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_table_proto_depIdxs,
		EnumInfos:         file_google_cloud_bigquery_storage_v1_table_proto_enumTypes,
		MessageInfos:      file_google_cloud_bigquery_storage_v1_table_proto_msgTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_table_proto = out.File
	file_google_cloud_bigquery_storage_v1_table_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_table_proto_depIdxs = nil
}
