// Code generated by ./cmd/ch-gen-col, DO NOT EDIT.

package proto

// ColDate32 represents Date32 column.
type ColDate32 []Date32

// Compile-time assertions for ColDate32.
var (
	_ ColInput  = ColDate32{}
	_ ColResult = (*ColDate32)(nil)
	_ Column    = (*ColDate32)(nil)
)

// Rows returns count of rows in column.
func (c ColDate32) Rows() int {
	return len(c)
}

// Reset resets data in row, preserving capacity for efficiency.
func (c *ColDate32) Reset() {
	*c = (*c)[:0]
}

// Type returns ColumnType of Date32.
func (ColDate32) Type() ColumnType {
	return ColumnTypeDate32
}
