// Code generated by ./cmd/ch-gen-col, DO NOT EDIT.

package proto

// ColEnum16 represents Enum16 column.
type ColEnum16 []Enum16

// Compile-time assertions for ColEnum16.
var (
	_ ColInput  = ColEnum16{}
	_ ColResult = (*ColEnum16)(nil)
	_ Column    = (*ColEnum16)(nil)
)

// Rows returns count of rows in column.
func (c ColEnum16) Rows() int {
	return len(c)
}

// Reset resets data in row, preserving capacity for efficiency.
func (c *ColEnum16) Reset() {
	*c = (*c)[:0]
}

// Type returns ColumnType of Enum16.
func (ColEnum16) Type() ColumnType {
	return ColumnTypeEnum16
}

// Row returns i-th row of column.
func (c ColEnum16) Row(i int) Enum16 {
	return c[i]
}

// Append Enum16 to column.
func (c *ColEnum16) Append(v Enum16) {
	*c = append(*c, v)
}

// Append Enum16 slice to column.
func (c *ColEnum16) AppendArr(vs []Enum16) {
	*c = append(*c, vs...)
}

// LowCardinality returns LowCardinality for Enum16.
func (c *ColEnum16) LowCardinality() *ColLowCardinality[Enum16] {
	return &ColLowCardinality[Enum16]{
		index: c,
	}
}

// Array is helper that creates Array of Enum16.
func (c *ColEnum16) Array() *ColArr[Enum16] {
	return &ColArr[Enum16]{
		Data: c,
	}
}

// Nullable is helper that creates Nullable(Enum16).
func (c *ColEnum16) Nullable() *ColNullable[Enum16] {
	return &ColNullable[Enum16]{
		Values: c,
	}
}

// NewArrEnum16 returns new Array(Enum16).
func NewArrEnum16() *ColArr[Enum16] {
	return &ColArr[Enum16]{
		Data: new(ColEnum16),
	}
}
