// Code generated by ./cmd/ch-gen-col, DO NOT EDIT.

package proto

func inferGenerated(t ColumnType) Column {
	switch t {
	case ColumnTypeFloat32:
		return new(ColFloat32)
	case ColumnTypeFloat64:
		return new(ColFloat64)
	case ColumnTypeIPv4:
		return new(ColIPv4)
	case ColumnTypeIPv6:
		return new(ColIPv6)
	case ColumnTypeDate:
		return new(ColDate)
	case ColumnTypeDate32:
		return new(ColDate32)
	case ColumnTypeTime32:
		return new(ColTime32)
	case ColumnTypeTime64:
		return new(ColTime64)
	case ColumnTypeInt8:
		return new(ColInt8)
	case ColumnTypeUInt8:
		return new(ColUInt8)
	case ColumnTypeInt16:
		return new(ColInt16)
	case ColumnTypeUInt16:
		return new(ColUInt16)
	case ColumnTypeInt32:
		return new(ColInt32)
	case ColumnTypeUInt32:
		return new(ColUInt32)
	case ColumnTypeInt64:
		return new(ColInt64)
	case ColumnTypeUInt64:
		return new(ColUInt64)
	case ColumnTypeInt128:
		return new(ColInt128)
	case ColumnTypeUInt128:
		return new(ColUInt128)
	case ColumnTypeInt256:
		return new(ColInt256)
	case ColumnTypeUInt256:
		return new(ColUInt256)
	case ColumnTypeFixedString.With("8"):
		return new(ColFixedStr8)
	case ColumnTypeFixedString.With("16"):
		return new(ColFixedStr16)
	case ColumnTypeFixedString.With("32"):
		return new(ColFixedStr32)
	case ColumnTypeFixedString.With("64"):
		return new(ColFixedStr64)
	case ColumnTypeFixedString.With("128"):
		return new(ColFixedStr128)
	case ColumnTypeFixedString.With("256"):
		return new(ColFixedStr256)
	case ColumnTypeFixedString.With("512"):
		return new(ColFixedStr512)
	default:
		return nil
	}
}
