// Code generated by ./cmd/ch-gen-col, DO NOT EDIT.

package proto

// ColDate represents Date column.
type ColDate []Date

// Compile-time assertions for ColDate.
var (
	_ ColInput  = ColDate{}
	_ ColResult = (*ColDate)(nil)
	_ Column    = (*ColDate)(nil)
)

// Rows returns count of rows in column.
func (c ColDate) Rows() int {
	return len(c)
}

// Reset resets data in row, preserving capacity for efficiency.
func (c *ColDate) Reset() {
	*c = (*c)[:0]
}

// Type returns ColumnType of Date.
func (ColDate) Type() ColumnType {
	return ColumnTypeDate
}
