// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.0
// source: google/cloud/bigquery/storage/v1/avro.proto

package storagepb

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The precision of the timestamp value in the Avro message. This precision
// will **only** be applied to the column(s) with the `TIMESTAMP_PICOS` type.
type AvroSerializationOptions_PicosTimestampPrecision int32

const (
	// Unspecified timestamp precision. The default precision is microseconds.
	AvroSerializationOptions_PICOS_TIMESTAMP_PRECISION_UNSPECIFIED AvroSerializationOptions_PicosTimestampPrecision = 0
	// Timestamp values returned by Read API will be truncated to microsecond
	// level precision. The value will be encoded as Avro TIMESTAMP type in a
	// 64 bit integer.
	AvroSerializationOptions_TIMESTAMP_PRECISION_MICROS AvroSerializationOptions_PicosTimestampPrecision = 1
	// Timestamp values returned by Read API will be truncated to nanosecond
	// level precision. The value will be encoded as Avro TIMESTAMP type in a
	// 64 bit integer.
	AvroSerializationOptions_TIMESTAMP_PRECISION_NANOS AvroSerializationOptions_PicosTimestampPrecision = 2
	// Read API will return full precision picosecond value. The value will be
	// encoded as a string which conforms to ISO 8601 format.
	AvroSerializationOptions_TIMESTAMP_PRECISION_PICOS AvroSerializationOptions_PicosTimestampPrecision = 3
)

// Enum value maps for AvroSerializationOptions_PicosTimestampPrecision.
var (
	AvroSerializationOptions_PicosTimestampPrecision_name = map[int32]string{
		0: "PICOS_TIMESTAMP_PRECISION_UNSPECIFIED",
		1: "TIMESTAMP_PRECISION_MICROS",
		2: "TIMESTAMP_PRECISION_NANOS",
		3: "TIMESTAMP_PRECISION_PICOS",
	}
	AvroSerializationOptions_PicosTimestampPrecision_value = map[string]int32{
		"PICOS_TIMESTAMP_PRECISION_UNSPECIFIED": 0,
		"TIMESTAMP_PRECISION_MICROS":            1,
		"TIMESTAMP_PRECISION_NANOS":             2,
		"TIMESTAMP_PRECISION_PICOS":             3,
	}
)

func (x AvroSerializationOptions_PicosTimestampPrecision) Enum() *AvroSerializationOptions_PicosTimestampPrecision {
	p := new(AvroSerializationOptions_PicosTimestampPrecision)
	*p = x
	return p
}

func (x AvroSerializationOptions_PicosTimestampPrecision) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AvroSerializationOptions_PicosTimestampPrecision) Descriptor() protoreflect.EnumDescriptor {
	return file_google_cloud_bigquery_storage_v1_avro_proto_enumTypes[0].Descriptor()
}

func (AvroSerializationOptions_PicosTimestampPrecision) Type() protoreflect.EnumType {
	return &file_google_cloud_bigquery_storage_v1_avro_proto_enumTypes[0]
}

func (x AvroSerializationOptions_PicosTimestampPrecision) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AvroSerializationOptions_PicosTimestampPrecision.Descriptor instead.
func (AvroSerializationOptions_PicosTimestampPrecision) EnumDescriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_avro_proto_rawDescGZIP(), []int{2, 0}
}

// Avro schema.
type AvroSchema struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Json serialized schema, as described at
	// https://avro.apache.org/docs/1.8.1/spec.html.
	Schema        string `protobuf:"bytes,1,opt,name=schema,proto3" json:"schema,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AvroSchema) Reset() {
	*x = AvroSchema{}
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AvroSchema) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AvroSchema) ProtoMessage() {}

func (x *AvroSchema) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AvroSchema.ProtoReflect.Descriptor instead.
func (*AvroSchema) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_avro_proto_rawDescGZIP(), []int{0}
}

func (x *AvroSchema) GetSchema() string {
	if x != nil {
		return x.Schema
	}
	return ""
}

// Avro rows.
type AvroRows struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Binary serialized rows in a block.
	SerializedBinaryRows []byte `protobuf:"bytes,1,opt,name=serialized_binary_rows,json=serializedBinaryRows,proto3" json:"serialized_binary_rows,omitempty"`
	// [Deprecated] The count of rows in the returning block.
	// Please use the format-independent ReadRowsResponse.row_count instead.
	//
	// Deprecated: Marked as deprecated in google/cloud/bigquery/storage/v1/avro.proto.
	RowCount      int64 `protobuf:"varint,2,opt,name=row_count,json=rowCount,proto3" json:"row_count,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AvroRows) Reset() {
	*x = AvroRows{}
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AvroRows) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AvroRows) ProtoMessage() {}

func (x *AvroRows) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AvroRows.ProtoReflect.Descriptor instead.
func (*AvroRows) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_avro_proto_rawDescGZIP(), []int{1}
}

func (x *AvroRows) GetSerializedBinaryRows() []byte {
	if x != nil {
		return x.SerializedBinaryRows
	}
	return nil
}

// Deprecated: Marked as deprecated in google/cloud/bigquery/storage/v1/avro.proto.
func (x *AvroRows) GetRowCount() int64 {
	if x != nil {
		return x.RowCount
	}
	return 0
}

// Contains options specific to Avro Serialization.
type AvroSerializationOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Enable displayName attribute in Avro schema.
	//
	// The Avro specification requires field names to be alphanumeric.  By
	// default, in cases when column names do not conform to these requirements
	// (e.g. non-ascii unicode codepoints) and Avro is requested as an output
	// format, the CreateReadSession call will fail.
	//
	// Setting this field to true, populates avro field names with a placeholder
	// value and populates a "displayName" attribute for every avro field with the
	// original column name.
	EnableDisplayNameAttribute bool `protobuf:"varint,1,opt,name=enable_display_name_attribute,json=enableDisplayNameAttribute,proto3" json:"enable_display_name_attribute,omitempty"`
	// Optional. Set timestamp precision option. If not set, the default precision
	// is microseconds.
	PicosTimestampPrecision AvroSerializationOptions_PicosTimestampPrecision `protobuf:"varint,2,opt,name=picos_timestamp_precision,json=picosTimestampPrecision,proto3,enum=google.cloud.bigquery.storage.v1.AvroSerializationOptions_PicosTimestampPrecision" json:"picos_timestamp_precision,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *AvroSerializationOptions) Reset() {
	*x = AvroSerializationOptions{}
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AvroSerializationOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AvroSerializationOptions) ProtoMessage() {}

func (x *AvroSerializationOptions) ProtoReflect() protoreflect.Message {
	mi := &file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AvroSerializationOptions.ProtoReflect.Descriptor instead.
func (*AvroSerializationOptions) Descriptor() ([]byte, []int) {
	return file_google_cloud_bigquery_storage_v1_avro_proto_rawDescGZIP(), []int{2}
}

func (x *AvroSerializationOptions) GetEnableDisplayNameAttribute() bool {
	if x != nil {
		return x.EnableDisplayNameAttribute
	}
	return false
}

func (x *AvroSerializationOptions) GetPicosTimestampPrecision() AvroSerializationOptions_PicosTimestampPrecision {
	if x != nil {
		return x.PicosTimestampPrecision
	}
	return AvroSerializationOptions_PICOS_TIMESTAMP_PRECISION_UNSPECIFIED
}

var File_google_cloud_bigquery_storage_v1_avro_proto protoreflect.FileDescriptor

const file_google_cloud_bigquery_storage_v1_avro_proto_rawDesc = "" +
	"\n" +
	"+google/cloud/bigquery/storage/v1/avro.proto\x12 google.cloud.bigquery.storage.v1\"$\n" +
	"\n" +
	"AvroSchema\x12\x16\n" +
	"\x06schema\x18\x01 \x01(\tR\x06schema\"a\n" +
	"\bAvroRows\x124\n" +
	"\x16serialized_binary_rows\x18\x01 \x01(\fR\x14serializedBinaryRows\x12\x1f\n" +
	"\trow_count\x18\x02 \x01(\x03B\x02\x18\x01R\browCount\"\x93\x03\n" +
	"\x18AvroSerializationOptions\x12A\n" +
	"\x1denable_display_name_attribute\x18\x01 \x01(\bR\x1aenableDisplayNameAttribute\x12\x8e\x01\n" +
	"\x19picos_timestamp_precision\x18\x02 \x01(\x0e2R.google.cloud.bigquery.storage.v1.AvroSerializationOptions.PicosTimestampPrecisionR\x17picosTimestampPrecision\"\xa2\x01\n" +
	"\x17PicosTimestampPrecision\x12)\n" +
	"%PICOS_TIMESTAMP_PRECISION_UNSPECIFIED\x10\x00\x12\x1e\n" +
	"\x1aTIMESTAMP_PRECISION_MICROS\x10\x01\x12\x1d\n" +
	"\x19TIMESTAMP_PRECISION_NANOS\x10\x02\x12\x1d\n" +
	"\x19TIMESTAMP_PRECISION_PICOS\x10\x03B\xb9\x01\n" +
	"$com.google.cloud.bigquery.storage.v1B\tAvroProtoP\x01Z>cloud.google.com/go/bigquery/storage/apiv1/storagepb;storagepb\xaa\x02 Google.Cloud.BigQuery.Storage.V1\xca\x02 Google\\Cloud\\BigQuery\\Storage\\V1b\x06proto3"

var (
	file_google_cloud_bigquery_storage_v1_avro_proto_rawDescOnce sync.Once
	file_google_cloud_bigquery_storage_v1_avro_proto_rawDescData []byte
)

func file_google_cloud_bigquery_storage_v1_avro_proto_rawDescGZIP() []byte {
	file_google_cloud_bigquery_storage_v1_avro_proto_rawDescOnce.Do(func() {
		file_google_cloud_bigquery_storage_v1_avro_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_avro_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_avro_proto_rawDesc)))
	})
	return file_google_cloud_bigquery_storage_v1_avro_proto_rawDescData
}

var file_google_cloud_bigquery_storage_v1_avro_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_google_cloud_bigquery_storage_v1_avro_proto_goTypes = []any{
	(AvroSerializationOptions_PicosTimestampPrecision)(0), // 0: google.cloud.bigquery.storage.v1.AvroSerializationOptions.PicosTimestampPrecision
	(*AvroSchema)(nil),               // 1: google.cloud.bigquery.storage.v1.AvroSchema
	(*AvroRows)(nil),                 // 2: google.cloud.bigquery.storage.v1.AvroRows
	(*AvroSerializationOptions)(nil), // 3: google.cloud.bigquery.storage.v1.AvroSerializationOptions
}
var file_google_cloud_bigquery_storage_v1_avro_proto_depIdxs = []int32{
	0, // 0: google.cloud.bigquery.storage.v1.AvroSerializationOptions.picos_timestamp_precision:type_name -> google.cloud.bigquery.storage.v1.AvroSerializationOptions.PicosTimestampPrecision
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_google_cloud_bigquery_storage_v1_avro_proto_init() }
func file_google_cloud_bigquery_storage_v1_avro_proto_init() {
	if File_google_cloud_bigquery_storage_v1_avro_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_google_cloud_bigquery_storage_v1_avro_proto_rawDesc), len(file_google_cloud_bigquery_storage_v1_avro_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_google_cloud_bigquery_storage_v1_avro_proto_goTypes,
		DependencyIndexes: file_google_cloud_bigquery_storage_v1_avro_proto_depIdxs,
		EnumInfos:         file_google_cloud_bigquery_storage_v1_avro_proto_enumTypes,
		MessageInfos:      file_google_cloud_bigquery_storage_v1_avro_proto_msgTypes,
	}.Build()
	File_google_cloud_bigquery_storage_v1_avro_proto = out.File
	file_google_cloud_bigquery_storage_v1_avro_proto_goTypes = nil
	file_google_cloud_bigquery_storage_v1_avro_proto_depIdxs = nil
}
